#!/usr/bin/perl
# -----------------------------------------------------------------------------
#   ident "@(#)ma_jdmk.pl 1.19 04/09/22 SMI"
# -----------------------------------------------------------------------------
#
#  Copyright 2001-2004 Sun Microsystems, Inc. All rights reserved.
#
# -----------------------------------------------------------------------------

package MA_JDMK;

use strict;
use variables;


# -----------------------------------------------------------------------------
#
# update the nma.conf file
#
# -----------------------------------------------------------------------------

sub update_ma_file
  {
    my ($ma_file, $update_entry_input, $update_value_input) = @_;
    
    my $count=0;
    my $update_entry;
    my $update_value;
    my $garbage;
    
    open (MAFILE, $ma_file ) || COMMON::error("Unable to open $ma_file");
    open (MAFILE_TMP, ">${ma_file}.tmp" ) || COMMON::error("Unable to open ${ma_file}.tmp");
    
    while ( <MAFILE> ) {
    
      chop();
      if ($_ eq "") {
	next;
      }

      ($update_entry, $update_value, $garbage) = split (/=/, $_) ;
      
      if ( "$update_entry" eq "$update_entry_input" ) {
	print MAFILE_TMP "${update_entry}=${update_value_input}\n";
	$count++;
      } else {
	print MAFILE_TMP "$_\n";
      }
      next;
      
    }
    close (MAFILE);
    close (MAFILE_TMP);
    
    unless ( $count ) {
      open (MAFILE_TMP, ">>${ma_file}.tmp" ) || COMMON::error("Unable to update $ma_file");
      print MAFILE_TMP "$update_entry_input=$update_value_input\n";
      close (MAFILE_TMP);
    }
    
    COMMON::exec_cmd ( "$MV ${ma_file}.tmp ${ma_file}") ||
      COMMON::error ( "Unable to move ${ma_file}.tmp ${ma_file}");
    }

# -----------------------------------------------------------------------------
#
#  configure nma.conf on local disk
#
# -----------------------------------------------------------------------------

sub configure_local_ma {

  my ($node_id) = @_ ;

  my $local_maconf = "$WORKING_DIR/nma.conf_${node_id}";
  my $remote_maconf = "/etc/opt/SUNWcgha/nma.conf";
  my $node_ip   = COMMON::get_external_ip($node_id);
  
  if ($USE_CGTP) {
    return ;
  }
  
  COMMON::print_action("Management agent configuration (no CGTP)");
  
  COMMON::get_file($node_ip, "$remote_maconf", "$local_maconf");
  
  update_ma_file($local_maconf, "NMA_NO_CGTP", "1");
  
  COMMON::put_file($node_ip, "$local_maconf", "$remote_maconf");

  }


#--------------------------------------------------------------
#
# installation of MA and JDMK packages on MEN
#
#--------------------------------------------------------------

sub men_ma_jdmk {

  ($NODE_ID) = @_;

  COMMON::print_stage_node("Management Agent (MA) and JDMK installation") ;

  COMPONENT::install_men_component($NODE_ID, "MA", "");
  COMPONENT::install_men_component($NODE_ID, "JDMK", "");

  configure_local_ma($NODE_ID);
  
  # Install shared packages MA and JDMK, and Supplementary packages on only first MEN
  
  if ( $NODE_ID eq $MEN1_ID ) {
    
    COMPONENT::shared_patch_preparation();
    COMPONENT::install_men_component($NODE_ID, "MA_SHARED", "");
    COMPONENT::install_men_component($NODE_ID, "JDMK_SHARED", "");
  }
}

#--------------------------------------------------------------
#
# recovery after failure on MEN
#
#--------------------------------------------------------------

sub recovery_men_ma_jdmk {

  ($NODE_ID) = @_;
  
  if ($NODE_ID eq $MEN1_ID) {
    COMMON::print_recovery_node("Management Agent (MA) and JDMK clean-up") ;
    COMPONENT::remove_men_component($NODE_ID, "JDMK_SHARED", "");
  }
}
#--------------------------------------------------------------
#
# installation of MA and JDMK packages on MEN
#
#--------------------------------------------------------------

sub dataless_ma_jdmk {

  ($NODE_ID) = @_;

  COMMON::print_stage_node("Management Agent (MA) and JDMK installation for dataless") ;

  COMPONENT::install_dataless_component($NODE_ID, "MA", "");
  COMPONENT::install_dataless_component($NODE_ID, "JDMK", "");

  configure_local_ma($NODE_ID);
}
# -----------------------------------------------------------------------------
#
#  configure nma.conf for diskless
#
# -----------------------------------------------------------------------------

sub configure_diskless_ma {

  my ($node_id) = @_ ;
  my $maconf = "/etc/opt/SUNWcgha/nma.conf";
  my $node_ip = COMMON::get_external_ip($node_id);
  
  if ($USE_CGTP) {
    return ;
  }
  
  COMMON::print_action("Management agent configuration (no CGTP)");
  
  for my $node (0..$#NODE_LIST) {
    
    if (($NODE_LIST[$node]{type} == $DISKLESS_NODE) && ($NODE_LIST[$node]{concerned})){
      
      my $node_name_template = COMMON::build_node_name($NODE_LIST[$node]{id}, "");
      COMMON::print_subaction("for $node_name_template") ;
      
      #
      # copy the version located on MEN1 in each diskless directory
      #
      COMMON::remote_exec ( $node_ip, "$RCOPY $maconf /export/root/${node_name_template}${maconf}") ||
	COMMON::error ( "Unable to copy $maconf to /export/root/${node_name_template}${maconf}");
    }
  }
}


#------------------------------------------------------------------------------
#
#  installation and configuration of MA and JDMK on diskless
#
#------------------------------------------------------------------------------

sub diskless_ma_jdmk {

  ($NODE_ID) = @_;

  # diskless environment is only set on the first MEN

  if ( $NODE_ID eq $MEN1_ID ) {
    COMMON::print_stage_node("Management Agent (MA) installation for diskless") ;

    COMPONENT::install_diskless_component($NODE_ID, "MA", "");
    COMPONENT::install_diskless_component($NODE_ID, "JDMK", "");

    configure_diskless_ma($NODE_ID);
  }
} 

{
}







