#!/usr/bin/perl
# -----------------------------------------------------------------------------
#  ident "@(#)conf.pl 1.69 04/10/11 SMI"
# -----------------------------------------------------------------------------
#
#  Copyright 2001-2004 Sun Microsystems, Inc. All rights reserved.
#
# -----------------------------------------------------------------------------

package CONF;

use strict;

use variables;
require "$ENV{NHINSTALL_LIB}/tools/lib/common.pl";
require "$ENV{NHINSTALL_LIB}/tools/lib/parser.pl";

#-----------------------------------------------------------------------------
#
# general variables
#
#-----------------------------------------------------------------------------

my $G_error = $FALSE;

my $DISKLESS_TYPE="CLUSTER";

my $new_node_syntax = $FALSE;
my $old_node_syntax = $FALSE;

#-----------------------------------------------------------------------------
#
# definition of the commands
#
#-----------------------------------------------------------------------------

#
# command definitions for cluster_definition.conf
#

#
# arguments common to NODE, MEN, DISKLESS, DATALESS
#
my @G_node_argument =
(
		{
		 meaning   => "node id",
		 name      => "id",
		 type      => $PARSER::CMD_INTEGER,
		 field     => "id",
		 mandatory => $TRUE,
		 control   => "1..",
		},
		{
		 meaning  => "NIC 0 Mac Address or client id",
		 name      => "mac0",
		 type      => $PARSER::CMD_CLIENT_ID,
		 field     => "eth0",
		 mandatory => $FALSE,
		 control   => "",
		} ,
		{
		 meaning   => "NIC 1 Mac Address",
		 name      => "mac1",
		 type      => $PARSER::CMD_CLIENT_ID,
		 field     => "eth1",
		 mandatory => $FALSE,
		 control   => "",
		} ,
		{
		 meaning   => "CGTP host name",
		 name      => "hostname",
		 type      => $PARSER::CMD_HOST,
		 field     => "name",
		 mandatory => $FALSE,
		 control   => $PARSER::CMD_FULL_HOST,
		} ,
		{
		 meaning   => "CGTP nic0",
		 name      => "nic0",
		 type      => $PARSER::CMD_NIC,
		 field     => "nic0",
		 mandatory => $FALSE,
		 control   => "",
		} ,
		{
		 meaning   => "CGTP nic1",
		 name      => "nic1",
		 type      => $PARSER::CMD_NIC,
		 field     => "nic1",
		 mandatory => $FALSE,
		 control   => "",
		} ,
		{
		 meaning   => "public host name",
		 name      => "pubname",
		 type      => $PARSER::CMD_HOST,
		 field     => "pubName",
		 mandatory => $FALSE,
		 control   => $PARSER::CMD_FULL_HOST,
		} ,
		{
		 meaning   => "public IP",
		 name      => "pubip",
		 type      => $PARSER::CMD_DOTTED_IP,
		 field     => "pubIp",
		 mandatory => $FALSE,
		 control   => "",
		} ,
		{
		 meaning   => "public NIC",
		 name      => "pubnic",
		 type      => $PARSER::CMD_NIC,
		 field     => "pubNic",
		 mandatory => $FALSE,
		 control   => "",
		} ,
);

my @G_cluster_command =
(
 # CLUSTER_ID
 { name     => "CLUSTER_ID",
   category => $PARSER::CMD_SIMPLE,
   argument => [
		{
		 meaning   => "cluster id",
		 name      => "",
		 type      => $PARSER::CMD_INTEGER,
		 variable  => \$CLUSTER,
		 mandatory => $TRUE,
		 control   => "1..255",
		}
	       ],
 },

 # ARCH
 { name     => "ARCH",
   category => $PARSER::CMD_SIMPLE,
   argument => [
		{
		 meaning   => "architecture",
		 name      => "",
		 type      => $PARSER::CMD_STRING,
		 variable  => \$ARCH,
		 mandatory => $TRUE,
		 control   => "",
		}
	       ],
 },

 # MEN_INTERFACES
 { name     => "MEN_INTERFACES",
   category => $PARSER::CMD_FIELD,
   argument => [
		{
		 meaning   => "nic0",
		 name      => "nic0",
		 type      => $PARSER::CMD_NIC,
		 variable  => \$MEN_NIC0,
		 mandatory => $TRUE,
		 control   => "",
		},
		{
		 meaning   => "nic1",
		 name      => "nic1",
		 type      => $PARSER::CMD_NIC,
		 variable  => \$MEN_NIC1,
		 mandatory => $FALSE,
		 control   => "",
		}
	       ],
 },

 # NMEN_INTERFACES
 { name     => "NMEN_INTERFACES",
   category => $PARSER::CMD_FIELD,
   argument => [
		{
		 meaning   => "nic0",
		 name      => "nic0",
		 type      => $PARSER::CMD_NIC,
		 variable  => \$NMEN_NIC0,
		 mandatory => $TRUE,
		 control   => "",
		},
		{
		 meaning   => "nic1",
		 name      => "nic1",
		 type      => $PARSER::CMD_NIC,
		 variable  => \$NMEN_NIC1,
		 mandatory => $FALSE,
		 control   => "",
		}
	       ],
 },

 # NIC0_POSTFIX
 { name     => "NIC0_POSTFIX",
   category => $PARSER::CMD_SIMPLE,
   argument => [
		{
		 meaning  => "NIC0 postfix",
		 name     => "",
		 type     => $PARSER::CMD_HOST,
		 variable => \$POST_NIC0,
		 mandatory => $FALSE, # can be an empty string
		 control   => $PARSER::CMD_PARTIAL_HOST,
		}
	       ],
 } ,

 # NIC1_POSTFIX
 { name     => "NIC1_POSTFIX",
   category => $PARSER::CMD_SIMPLE,
   argument => [
		{
		 meaning  => "NIC1 postfix",
		 name     => "",
		 type     => $PARSER::CMD_HOST,
		 variable => \$POST_NIC1,
		 mandatory => $FALSE, # can be an empty string
		 control   => $PARSER::CMD_PARTIAL_HOST,
		}
	       ],
 } ,

 # CGTO_POSTFIX
 { name     => "CGTP_POSTFIX",
   category => $PARSER::CMD_SIMPLE,
   argument => [
		{
		 meaning  => "NIC0 postfix",
		 name     => "",
		 type     => $PARSER::CMD_HOST,
		 variable => \$POST_CGTP,
		 mandatory => $FALSE, # can be an empty string
		 control   => $PARSER::CMD_PARTIAL_HOST,
		}
	       ],
 } ,

 # NODE
 { name     => "NODE",
   category => $PARSER::CMD_HASH,
   argument => \@G_node_argument,
   array    => \@NODE_LIST,
   post     => \&control_node_item,
 } ,

 # MEN
 { name     => "MEN",
   category => $PARSER::CMD_HASH,
   argument => \@G_node_argument,
   array    => \@NODE_LIST,
   post     => \&control_men_item,
 } ,

 # DISKLESS
 { name     => "DISKLESS",
   category => $PARSER::CMD_HASH,
   argument => \@G_node_argument,
   array    => \@NODE_LIST,
   post     => \&control_diskless_item,
 } ,

 # DATALESS
 { name     => "DATALESS",
   category => $PARSER::CMD_HASH,
   argument => \@G_node_argument,
   array    => \@NODE_LIST,
   post     => \&control_dataless_item,
 } ,

 # VENDOR_TYPE
 { name     => "VENDOR_TYPE",
   category => $PARSER::CMD_ARRAY,
   argument => [
		{
		 meaning   => "vendor type",
		 name      => "",
		 type      => $PARSER::CMD_VENDOR,
		 mandatory => $TRUE,
		 control   => "",
		}
	       ],
   array    => \@VENDOR_TYPE,
 } ,

 # DISKLESS_BOOT_POLICY
 { name     => "DISKLESS_BOOT_POLICY",
   category => $PARSER::CMD_SIMPLE,
   argument => [
		{
		 meaning   => "diskless boot policy",
		 name      => "",
		 type      => $PARSER::CMD_LIST,
		 variable  => \$DISKLESS_BOOT_POLICY,
		 mandatory => $TRUE,
		 control   => "DHCP_DYNAMIC,DHCP_STATIC,DHCP_CLIENT_ID",
		}
	       ],
 } ,

 # DISKLESS_TYPE
 { name     => "DISKLESS_TYPE",
   category => $PARSER::CMD_SIMPLE,
   argument => [
		{
		 meaning   => "diskless type",
		 name      => "",
		 type      => $PARSER::CMD_LIST,
		 variable  => \$DISKLESS_TYPE,
		 mandatory => $TRUE,
		 control   => "CLUSTER,STANDALONE",
		}
	       ],
 } ,

 # REPLICATED_DHCP_FILES
 { name     => "REPLICATED_DHCP_FILES",
   category => $PARSER::CMD_SIMPLE,
   argument => [
		{
		 meaning   => "Replication of DHCP configuration files",
		 name      => "",
		 type      => $PARSER::CMD_BOOLEAN,
		 variable  => \$REPLICATED_DHCP_FILES,
		 mandatory => $TRUE,
		 control   => "",
		}
	       ],
 } ,

 # USE_CGTP
 { name     => "USE_CGTP",
   category => $PARSER::CMD_SIMPLE,
   argument => [
		{
		 meaning   => "CGTP",
		 name      => "",
		 type      => $PARSER::CMD_BOOLEAN,
		 variable  => \$USE_CGTP,
		 mandatory => $TRUE,
		 control   => "",
		}
	       ],
 } ,

 # USE_WDT
 { name     => "USE_WDT",
   category => $PARSER::CMD_SIMPLE,
   argument => [
		{
		 meaning   => "watchdog",
		 name      => "",
		 type      => $PARSER::CMD_BOOLEAN,
		 variable  => \$USE_WDT,
		 mandatory => $TRUE,
		 control   => "",
		}
	       ],
 } ,

 # RESTRICT_RHOSTS
 { name     => "RESTRICT_RHOSTS",
   category => $PARSER::CMD_SIMPLE,
   argument => [
		{
		 meaning   => "restricted .rhosts",
		 name      => "",
		 type      => $PARSER::CMD_BOOLEAN,
		 variable  => \$RESTRICT_RHOSTS,
		 mandatory => $TRUE,
		 control   => "",
		}
	       ],
 } ,

 # BITMAP_IN_MEMORY
 { name     => "BITMAP_IN_MEMORY",
   category => $PARSER::CMD_SIMPLE,
   argument => [
		{
		 meaning   => "SNDR bitmap in memory",
		 name      => "",
		 type      => $PARSER::CMD_BOOLEAN,
		 variable  => \$BITMAP_IN_MEMORY,
		 mandatory => $TRUE,
		 control   => "",
		}
	       ],
 } ,

 # NFS_USER_DIR_NOAC
 { name     => "NFS_USER_DIR_NOAC",
   category => $PARSER::CMD_SIMPLE,
   argument => [
		{
		 meaning   => "NFS user's dir noac",
		 name      => "",
		 type      => $PARSER::CMD_BOOLEAN,
		 variable  => \$RNFS_NOAC,
		 mandatory => $TRUE,
		 control   => "",
		}
	       ],
 } ,

 # LOGICAL_SLICE_SUPPORT
 { name     => "LOGICAL_SLICE_SUPPORT",
   category => $PARSER::CMD_SIMPLE,
   argument => [
		{
		 meaning   => "SVM on top of replicated slice",
		 name      => "",
		 type      => $PARSER::CMD_BOOLEAN,
		 variable  => \$USE_SVM,
		 mandatory => $TRUE,
		 control   => "",
		}
	       ],
 } ,

 # IDE_SUPPORT
 { name     => "IDE_SUPPORT",
   category => $PARSER::CMD_SIMPLE,
   argument => [
		{
		 meaning   => "support of IDE disk",
		 name      => "",
		 type      => $PARSER::CMD_BOOLEAN,
		 variable  => \$IDE_SUPPORT,
		 mandatory => $TRUE,
		 control   => "",
		}
	       ],
 } ,

 # USE_DCSS
 { name     => "USE_DCSS",
   category => $PARSER::CMD_SIMPLE,
   argument => [
		{
		 meaning   => "use of SUNWdcss",
		 name      => "",
		 type      => $PARSER::CMD_BOOLEAN,
		 variable  => \$USE_DCSS,
		 mandatory => $TRUE,
		 control   => "",
		}
	       ],
 } ,

 # PATCH_WITH_SMOSSERVICE
 { name     => "PATCH_WITH_SMOSSERVICE",
   category => $PARSER::CMD_SIMPLE,
   argument => [
		{
		 meaning   => "use of smosservice for diskless patching",
		 name      => "",
		 type      => $PARSER::CMD_BOOLEAN,
		 variable  => \$PATCH_WITH_SMOSSERVICE,
		 mandatory => $TRUE,
		 control   => "",
		}
	       ],
 } ,

 # SLICE_SYNC_TYPE
 { name     => "SLICE_SYNC_TYPE",
   category => $PARSER::CMD_SIMPLE,
   argument => [
		{
		 meaning   => "slice synchronization type",
		 name      => "",
		 type      => $PARSER::CMD_LIST,
		 variable  => \$SYNC_TYPE,
		 mandatory => $TRUE,
		 control   => "FS,RAW",
		}
	       ],
 } ,

 # CHECK_REPLICATED_SLICES
 { name     => "CHECK_REPLICATED_SLICES",
   category => $PARSER::CMD_SIMPLE,
   argument => [
		{
		 meaning   => "sanity check of replicated slices",
		 name      => "",
		 type      => $PARSER::CMD_BOOLEAN,
		 variable  => \$CHECK_REPLICATED_SLICES,
		 mandatory => $FALSE,
		 control   => "",
		}
	       ],
 } ,

 # MASTER_LOSS_DETECTION
 { name     => "MASTER_LOSS_DETECTION",
   category => $PARSER::CMD_SIMPLE,
   argument => [
		{
		 meaning   => "detection of master loss on diskless",
		 name      => "",
		 type      => $PARSER::CMD_BOOLEAN,
		 variable  => \$MASTER_LOSS_DETECTION,
		 mandatory => $FALSE,
		 control   => "",
		}
	       ],
 } ,

 # PASSWORD
 { name     => "PASSWORD",
   category => $PARSER::CMD_SIMPLE,
   argument => [
		{
		 meaning   => "root password",
		 name      => "",
		 type      => $PARSER::CMD_STRING,
		 variable  => \$PASSWORD,
		 mandatory => $TRUE,
		 control   => "",
		}
	       ],
 } ,

 # EXPORTED
 { name     => "EXPORTED",
   category => $PARSER::CMD_ARRAY,
   argument => [
		{
		 meaning   => "exported directory",
		 name      => "dir",
		 type      => $PARSER::CMD_STRING,
		 mandatory => $TRUE,
		 control   => "",
		}
	       ],
   array    => \@DIR_EXPORTED,
 } ,

 # MOUNTED
 { name     => "MOUNTED",
   category => $PARSER::CMD_HASH,
   argument => [
		{
		 meaning   => "local directory",
		 name      => "local",
		 type      => $PARSER::CMD_STRING,
		 field     => "local",
		 mandatory => $TRUE,
		 control   => "",
		} ,
		{
		 meaning   => "remote directory",
		 name      => "remote",
		 type      => $PARSER::CMD_STRING,
		 field     => "remote",
		 mandatory => $TRUE,
		 control   => "",
		}
	       ],
   array    => \@DIR_MOUNTED,
 } ,

 # SLICE
 { name     => "SLICE",
   category => $PARSER::CMD_HASH,
   argument => [
		{
		 meaning   => "slice",
		 name      => "slice",
		 type      => $PARSER::CMD_STRING,
		 field     => "name",
		 mandatory => $TRUE,
		 control   => "",
		} ,
		{
		 meaning   => "size",
		 name      => "size",
		 type      => $PARSER::CMD_STRING,
		 field     => "size",
		 mandatory => $TRUE,
		 control   => "",
		} ,
		{
		 meaning   => "mounting point",
		 name      => "mount",
		 type      => $PARSER::CMD_STRING,
		 field     => "mounting_point",
		 mandatory => $TRUE,
		 control   => "",
		} ,
		{
		 meaning   => "bitmap slice",
		 name      => "bitmap",
		 type      => $PARSER::CMD_STRING,
		 field     => "bitmap",
		 mandatory => $FALSE,
		 control   => "",
		} ,
		{
		 meaning   => "mount option",
		 name      => "option",
		 type      => $PARSER::CMD_STRING,
		 field     => "option",
		 mandatory => $FALSE,
		 control   => "",
		} ,
		{
		 meaning   => "scope",
		 name      => "scope",
		 type      => $PARSER::CMD_STRING,
		 field     => "scope",
		 mandatory => $FALSE,
		 control   => "",
		 default   => "MEN",
		}
	       ],
   array    => \@SLICE_LIST,
   post     => \&control_slice_item,
 } ,

 # CLUSTER_NETWORK
 { name     => "CLUSTER_NETWORK",
   category => $PARSER::CMD_FIELD,
   flag     => \$USE_CLUSTER_NETWORK,
   argument => [
		{
		 meaning   => "netmask",
		 name      => "netmask",
		 type      => $PARSER::CMD_DOTTED_IP,
		 variable  => \$CLUSTER_DOTTED_NETMASK,
		 mandatory => $TRUE,
		 control   => "",
		},
		{
		 meaning   => "nic0 subnet",
		 name      => "nic0net",
		 type      => $PARSER::CMD_DOTTED_IP,
		 variable  => \$CLUSTER_DOTTED_NIC0_NET,
		 mandatory => $TRUE,
		 control   => "",
		},
		{
		 meaning   => "nic1 subnet",
		 name      => "nic1net",
		 type      => $PARSER::CMD_DOTTED_IP,
		 variable  => \$CLUSTER_DOTTED_NIC1_NET,
		 mandatory => $FALSE,
		 control   => "",
		},
		{
		 meaning   => "cgtp subnet",
		 name      => "cgtpnet",
		 type      => $PARSER::CMD_DOTTED_IP,
		 variable  => \$CLUSTER_DOTTED_CGTP_NET,
		 mandatory => $FALSE,
		 control   => "",
		},
	       ],
 },

 # PUBLIC_NETWORK
 { name     => "PUBLIC_NETWORK",
   category => $PARSER::CMD_FIELD,
   flag     => \$USE_PUBLIC_NETWORK,
   argument => [
		{ 
		 meaning   => "netmask",
		 name      => "netmask",
		 type      => $PARSER::CMD_DOTTED_IP,
		 variable  => \$PUBLIC_DOTTED_NETMASK,
		 mandatory => $TRUE,
		 control   => "",
		} ,
		{ 
		 meaning   => "network",
		 name      => "network",
		 type      => $PARSER::CMD_DOTTED_IP,
		 variable  => \$PUBLIC_DOTTED_NETWORK,
		 mandatory => $TRUE,
		 control   => "",
		}
	       ],
 } ,

 # DEFAULT_ROUTER_IP
 { name     => "DEFAULT_ROUTER_IP",
   category => $PARSER::CMD_SIMPLE,
   argument => [
		{ 
		 meaning   => "IP address of default router",
		 name      => "default_router_IP",
		 type      => $PARSER::CMD_DOTTED_IP,
		 variable  => \$DEFAULT_ROUTER_IP,
		 mandatory => $TRUE,
		 control   => "",
		}
	       ],
 } ,

 # EXTERNAL_ACCESS
 { name     => "EXTERNAL_ACCESS",
   category => $PARSER::CMD_FIELD,
   flag     => \$USE_EXTERNAL_ACCESS,
   argument => [
		{
		 meaning   => "external host name",
		 name      => "hostname",
		 type      => $PARSER::CMD_HOST,
		 variable  => \$EXTERNAL_HOSTNAME,
		 mandatory => $TRUE,
		 control   => $PARSER::CMD_FULL_HOST,
		},
		{
		 meaning   => "external IP address",
		 name      => "ip",
		 type      => $PARSER::CMD_DOTTED_IP,
		 variable  => \$EXTERNAL_IP,
		 mandatory => $TRUE,
		 control   => "",
		},
		{
		 meaning   => "external nic",
		 name      => "nic",
		 type      => $PARSER::CMD_NIC,
		 variable  => \$EXTERNAL_NIC,
		 mandatory => $TRUE,
		 control   => "",
		},
	       ],
 },

 # DIRECT_LINK
 { name     => "DIRECT_LINK",
   category => $PARSER::CMD_FIELD,
   flag     => \$USE_DIRECT_LINK,
   argument => [
		{
		 meaning   => "MEN #1 serial device",
		 name      => "device_1",
		 type      => $PARSER::CMD_STRING,
		 variable  => \$DIRECT_LINK_DEVICE_1,
		 mandatory => $TRUE,
		 control   => "",
		},
		{
		 meaning   => "MEN #2 serial device",
		 name      => "device_2",
		 type      => $PARSER::CMD_STRING,
		 variable  => \$DIRECT_LINK_DEVICE_2,
		 mandatory => $TRUE,
		 control   => "",
		},
		{
		 meaning   => "speed",
		 name      => "speed",
		 type      => $PARSER::CMD_LIST,
		 variable  => \$DIRECT_LINK_SPEED,
		 mandatory => $TRUE,
		 control   => "38400,57600,76800,115200",
		},
		{
		 meaning   => "heartbeat",
		 name      => "heartbeat",
		 type      => $PARSER::CMD_INTEGER,
		 variable  => \$DIRECT_LINK_HEARTBEAT,
		 mandatory => $FALSE,
		 control   => "",
		 default   => 20,
		},
	      ],
 },

 # DATA_MGT_POLICY
 { name     => "DATA_MGT_POLICY",
   category => $PARSER::CMD_SIMPLE,
   argument => [
		{
		 meaning   => "cluster data management policy",
		 name      => "cluster data management policy",
		 type      => $PARSER::CMD_LIST,
		 variable  => \$DATA_MGT_POLICY,
		 mandatory => $FALSE,
		 control   => "AVAILABILITY,INTEGRITY,ADAPTABILITY",
		 default   => "INTEGRITY",
		}
	       ],
 } ,

 # RFE-4846399
 # SYNC_FLAG
 { name     => "SYNC_FLAG",
   category => $PARSER::CMD_SIMPLE,
   argument => [
		{
		 meaning   => "SNDR sync flag",
		 name      => "",
		 type      => $PARSER::CMD_BOOLEAN,
		 variable  => \$SYNC_FLAG,
		 mandatory => $TRUE,
		 control   => "",
		}
	       ],
 } ,
 # RFE-4846399

 # RFE 4846389
 { name     => "SERIALIZE_SYNC",
   category => $PARSER::CMD_SIMPLE,
   argument => [
		{
		 meaning   => "SNDR serialize sync",
		 name      => "",
		 type      => $PARSER::CMD_BOOLEAN,
		 variable  => \$SERIALIZE_SYNC,
		 mandatory => $TRUE,
		 control   => "",
		}
	       ],
 } ,
 # RFE 4846389

 # MEN_OS_REFERENCE
 { name     => "MEN_OS_REFERENCE",
   category => $PARSER::CMD_SIMPLE,
   argument => [
		{
		 meaning   => "OS selection for MEN",
		 name      => "MEN_os_reference",
		 type      => $PARSER::CMD_STRING,
		 variable  => \$MEN_OS_REFERENCE,
		 mandatory => $TRUE,
		 control   => "",
		}
	       ],
 } ,

 # DATALESS_OS_REFERENCE
 { name     => "DATALESS_OS_REFERENCE",
   category => $PARSER::CMD_SIMPLE,
   argument => [
		{
		 meaning   => "OS selection for dataless",
		 name      => "dataless_os_reference",
		 type      => $PARSER::CMD_STRING,
		 variable  => \$DATALESS_OS_REFERENCE,
		 mandatory => $TRUE,
		 control   => "",
		}
	       ],
 } ,

 # DISKLESS_OS_REFERENCE
 { name     => "DISKLESS_OS_REFERENCE",
   category => $PARSER::CMD_SIMPLE,
   argument => [
		{
		 meaning   => "OS selection for diskless",
		 name      => "diskless_os_reference",
		 type      => $PARSER::CMD_STRING,
		 variable  => \$DISKLESS_OS_REFERENCE,
		 mandatory => $TRUE,
		 control   => "",
		}
	       ],
 } ,
);

#
# command definitions for addon.conf
#
my @G_package_argument =

(		{ 
		 meaning   => "reference",
		 name      => "reference",
		 type      => $PARSER::CMD_STRING,
		 field     => "reference",
		 mandatory => $TRUE,
		 control   => "",
		} ,
		{ 
		 meaning   => "exported directory",
		 name      => "dir",
		 type      => $PARSER::CMD_STRING,
		 field     => "dir",
		 mandatory => $TRUE,
		 control   => "",
		} ,
		{ 
		 meaning   => "sub-directory",
		 name      => "subdir",
		 type      => $PARSER::CMD_STRING,
		 field     => "subdir",
		 mandatory => $FALSE,
		 control   => "",
		} ,
		{ 
		 meaning   => "phase",
		 name      => "phase",
		 type      => $PARSER::CMD_LIST,
		 field     => "phase",
		 mandatory => $TRUE,
		 control   => "I,S,F",
		} ,
		{ 
		 meaning   => "scope",
		 name      => "scope",
		 type      => $PARSER::CMD_LIST,
		 field     => "scope",
		 mandatory => $TRUE,
		 control   => "LOCAL,SHARED,USR_SPECIFIC,USR_SOLARIS,CLONE_OPT",
		} ,
		{ 
		 meaning   => "men",
		 name      => "men",
		 type      => $PARSER::CMD_BOOLEAN_YN,
		 field     => "men",
		 mandatory => $FALSE,
		 control   => "",
		} ,
		{ 
		 meaning   => "diskless",
		 name      => "diskless",
		 type      => $PARSER::CMD_BOOLEAN_YN,
		 field     => "diskless",
		 mandatory => $FALSE,
		 control   => "",
		} ,
		{ 
		 meaning   => "dataless",
		 name      => "dataless",
		 type      => $PARSER::CMD_BOOLEAN_YN,
		 field     => "dataless",
		 mandatory => $FALSE,
		 control   => "",
		} ,
		{ 
		 meaning   => "installation method",
		 name      => "method",
		 type      => $PARSER::CMD_LIST,
		 field     => "method",
		 mandatory => $FALSE,
		 control   => "DEFAULT,STANDARD",
		} ,
);

my @G_patch_argument =

(		{ 
		 meaning   => "reference",
		 name      => "reference",
		 type      => $PARSER::CMD_STRING,
		 field     => "reference",
		 mandatory => $TRUE,
		 control   => "",
		} ,
		{ 
		 meaning   => "exported directory",
		 name      => "dir",
		 type      => $PARSER::CMD_STRING,
		 field     => "dir",
		 mandatory => $TRUE,
		 control   => "",
		} ,
		{ 
		 meaning   => "sub-directory",
		 name      => "subdir",
		 type      => $PARSER::CMD_STRING,
		 field     => "subdir",
		 mandatory => $FALSE,
		 control   => "",
		} ,
		{ 
		 meaning   => "phase",
		 name      => "phase",
		 type      => $PARSER::CMD_LIST,
		 field     => "phase",
		 mandatory => $TRUE,
		 control   => "I,S,F",
		} ,
		{ 
		 meaning   => "scope",
		 name      => "scope",
		 type      => $PARSER::CMD_LIST,
		 field     => "scope",
		 mandatory => $TRUE,
		 control   => "LOCAL,SHARED,USR_SPECIFIC,USR_SOLARIS",
		} ,
		{ 
		 meaning   => "men",
		 name      => "men",
		 type      => $PARSER::CMD_BOOLEAN_YN,
		 field     => "men",
		 mandatory => $FALSE,
		 control   => "",
		} ,
		{ 
		 meaning   => "diskless",
		 name      => "diskless",
		 type      => $PARSER::CMD_BOOLEAN_YN,
		 field     => "diskless",
		 mandatory => $FALSE,
		 control   => "",
		} ,
		{
		 meaning   => "dataless",
		 name      => "dataless",
		 type      => $PARSER::CMD_BOOLEAN_YN,
		 field     => "dataless",
		 mandatory => $FALSE,
		 control   => "",
		} ,
		{ 
		 meaning   => "installation method",
		 name      => "method",
		 type      => $PARSER::CMD_LIST,
		 field     => "method",
		 mandatory => $FALSE,
		 control   => "DEFAULT,STANDARD,SMOSSERVICE,PATCH_WITH_PKGADD",
		} ,
);

my @G_addon_command =
(
 # PACKAGE
 { name     => "PACKAGE",
   category => $PARSER::CMD_HASH,
   argument => \@G_package_argument,
   array    => \@ADDON_LIST,
   post     => \&control_package_item,
 } ,
 # PATCH
 { name     => "PATCH",
   category => $PARSER::CMD_HASH,
   argument => \@G_patch_argument,
   array    => \@ADDON_LIST,
   post     => \&control_patch_item,
 } ,
);




#==============================================================================
#
# POST-ITEM PROCESSING
#
# USE THE PARSER ERROR FUNCTION
#
#==============================================================================

#------------------------------------------------------------------------------
#
#  check package/patch parameter
#
#------------------------------------------------------------------------------

sub check_addon_directive {

  my ($idx) = @_;

  my $directive  = $ADDON_LIST[$idx]{type};
  my $reference  = $ADDON_LIST[$idx]{reference};
  my $scope      = $ADDON_LIST[$idx]{scope};
  my $men        = $ADDON_LIST[$idx]{men};
  my $diskless   = $ADDON_LIST[$idx]{diskless};
  my $dataless   = $ADDON_LIST[$idx]{dataless};
  my $phase      = $ADDON_LIST[$idx]{phase};

  my $directive_name = "";
  if ($directive == $ADDON_PATCH) {
    $directive_name = "PATCH";
  } else {
    $directive_name = "PACKAGE";
  }

  # coherency
  if (($scope eq $SCOPE_LOCAL) || ($scope eq $SCOPE_USR_SPECIFIC)) {
    if ($men eq "") {
      PARSER::print_error_no("$directive_name $reference: men mandatory ") ;
    }
    if (($diskless eq "") && ($DISKLESS_COUNT > 0)) {
      PARSER::print_error_no("$directive_name $reference: diskless mandatory");
    }
    if (($dataless eq "") && ($DATALESS_COUNT > 0)) {
      PARSER::print_error_no("$directive_name $reference: dataless mandatory");
    }
  }

  if (($scope eq $SCOPE_SHARED) 
      && (($phase eq $PHASE_I) || ($phase eq $PHASE_S))) {
    PARSER::print_error_no("$directive_name $reference: can be installed on the shared directory only after the installation of the Foundation Service on MEN");
  }

  my $method = $ADDON_LIST[$idx]{method};

  if ($ADDON_LIST[$idx]{type} == $ADDON_PATCH) {

    # patch

    if (($method eq "") || ($method eq "DEFAULT")) {
      $method = $STANDARD_METHOD;
      if ($PATCH_WITH_SMOSSERVICE && ($phase ne "F")) {
	$method = $SMOSSERVICE_METHOD;
      }
    } elsif ($method eq "STANDARD") {
      $method = $STANDARD_METHOD;
    } elsif ($method eq "SMOSSERVICE") {
      $method = $SMOSSERVICE_METHOD;
    } elsif ($method eq "PATCH_WITH_PKGADD") {
      $method = $PATCH_WITH_PKGADD_METHOD;
    }

  } else {

    # package
    my $method = $ADDON_LIST[$idx]{method};
    if (($method eq "") || ($method eq "DEFAULT")) {
      $method = $STANDARD_METHOD;
    }
  }
   
  $ADDON_LIST[$idx]{method} = $method;
}

#------------------------------------------------------------------------------
#
#  control_package_item
#
#------------------------------------------------------------------------------

sub control_package_item {

  my ($idx) = @_;

  $ADDON_LIST[$idx]{type} = $ADDON_PACKAGE;
  check_addon_directive($idx);
}

#------------------------------------------------------------------------------
#
#  control_patch_item
#
#------------------------------------------------------------------------------

sub control_patch_item {

  my ($idx) = @_;

  $ADDON_LIST[$idx]{type} = $ADDON_PATCH;
  check_addon_directive($idx);


}

#------------------------------------------------------------------------------
#
#  control_node_item
#
#------------------------------------------------------------------------------

sub control_node_item {

  my ($idx) = @_;

  if ($new_node_syntax) {
    PARSER::print_error_no("NODE definition: cannot be mixed with new node definition method");
  }
  $old_node_syntax = $TRUE;
  control_generic_node_item($idx, $UNKNOWN_NODE_TYPE);
}

#------------------------------------------------------------------------------
#
#  control_men_item
#
#------------------------------------------------------------------------------

sub control_men_item {

  my ($idx) = @_;

  if ($old_node_syntax) {
    PARSER::print_error_no("MEN definition: cannot be mixed with old node definition method");
  }
  $new_node_syntax = $TRUE;

  $MEN_COUNT++;
  control_generic_node_item($idx, $MEN_NODE);

}

#------------------------------------------------------------------------------
#
#  control_diskless_item
#
#------------------------------------------------------------------------------

sub control_diskless_item {

  my ($idx) = @_;

  if ($old_node_syntax) {
    PARSER::print_error_no("DISKLESS definition: cannot be mixed with old node definition method");
  }
  $new_node_syntax = $TRUE;

  $DISKLESS_COUNT++;
  control_generic_node_item($idx, $DISKLESS_NODE);

}

#------------------------------------------------------------------------------
#
#  control_dataless_item
#
#------------------------------------------------------------------------------

sub control_dataless_item {

  my ($idx) = @_;

  if ($old_node_syntax) {
    PARSER::print_error_no("DATALESS definition: cannot be mixed with old node definition method");
  }
  $new_node_syntax = $TRUE;

  $DATALESS_COUNT++;
  control_generic_node_item($idx, $DATALESS_NODE);

}

#------------------------------------------------------------------------------
#
#  control_node_item
#
#------------------------------------------------------------------------------

sub control_generic_node_item {

  my ($idx, $node_type) = @_;
  my $i;
  my $men_count;

  my $nodeid = $NODE_LIST[$idx]{id};

  $NODE_LIST[$idx]{concerned} = $FALSE;
  $NODE_LIST[$idx]{slice} = ();

  # control that the node id is not already used
  # the node id of the other nodes may be not defined because of errors
  for ($i = 0; $i < $idx; $i++) {
    if ($NODE_LIST[$i]{id} ne "") {
      if ($NODE_LIST[$i]{id} == $nodeid) {
	PARSER::print_error_no("Node id $nodeid: defined twice");
      }
    }
  }

  #
  # unknow node type
  #
  if ($node_type == $UNKNOWN_NODE_TYPE) {
    if (($idx == $MEN1) || ($idx == $MEN2)) {
      
      # it is a master eligible node
      if ($NODE_LIST[$idx]{eth0} eq "") {
	PARSER::print_error_no("NODE id $nodeid: MAC address required") ;
      }
      
      $MEN_COUNT++;
      $NODE_LIST[$idx]{type} = $MEN_NODE;
      
      if ($idx == $MEN1) {
	$MEN1_ID = $NODE_LIST[$idx]{id};
      } elsif ($idx == $MEN2) {
	$MEN2_ID = $NODE_LIST[$idx]{id} ;
      }
      
    } else {
      
      # this is a diskless node
      
      $DISKLESS_COUNT++;

      $NODE_LIST[$idx]{type} = $DISKLESS_NODE;
      $NODE_LIST[$idx]{concerned} = $FALSE;
      
    }

  } else {

    # well-known node type

    $NODE_LIST[$idx]{type} = $node_type;

    # check that is only two MENs
    # set MEN1_ID and MEN2_ID at the same time


    if ($node_type == $MEN_NODE) {

      if ($MEN_COUNT > 2) {
	PARSER::print_error_no("Node id $nodeid: only two master-eligible nodes are allowed");
      }

      if ($MEN_COUNT == 1) {
	$MEN1_ID = $NODE_LIST[$i]{id};
      } elsif ($MEN_COUNT == 2) {
	$MEN2_ID = $NODE_LIST[$i]{id};
      }
    }
  }
}

#------------------------------------------------------------------------------
#
#  control_slice_item
#
#------------------------------------------------------------------------------

sub control_slice_item {

  my ($idx) = @_;

  # initial state
  $SLICE_LIST[$idx]{metadb} = $FALSE;

  # check the coherency of the information
  # and update the calculated ones
  if (($SLICE_LIST[$idx]{size} eq "free") 
      && ($SLICE_LIST[$idx]{bitmap} ne "")) {
    PARSER::print_error_no("slice $SLICE_LIST[$idx]{name}: exact size required for a replicated slice (bitmap slice defined)") ;
  }
  
  # minimum size for a named slice
  if (($SLICE_LIST[$idx]{size} ne "free")
      && ($SLICE_LIST[$idx]{mounting_point} ne $SLICE_UNMOUNTED)) {
    if ($SLICE_LIST[$idx]{size} < $SLICE_MIN_SIZE) {
      PARSER::print_error_no("slice $SLICE_LIST[$idx]{name}: size must be at least $SLICE_MIN_SIZE Mbytes") ;
    }
  }
  
  # determine raw and block device name
  my $slice_name = $SLICE_LIST[$idx]{name};
  if ($slice_name =~ m/c[0-9]*t[0-9]*d[0-9]s[0-9]*/) {
    # if the slice looks like a physical slice
    $SLICE_LIST[$idx]{raw} = "/dev/rdsk/$slice_name";
    $SLICE_LIST[$idx]{block} = "/dev/dsk/$slice_name";
  } else {
    # probably a metadevice (usage of SVM other that automatically
    # through the installer for customer's specific configuration
    $SLICE_LIST[$idx]{raw} = "/dev/md/rdsk/$slice_name";
    $SLICE_LIST[$idx]{block} = "/dev/md/dsk/$slice_name";
  }
  $SLICE_LIST[$idx]{metadevice} = "";

  
}


#==============================================================================
#
#  ERROR PROCESSING
#
#==============================================================================

#------------------------------------------------------------------------------
#
# print_error
#
#------------------------------------------------------------------------------

sub print_error {
  my ($message) = @_;
  
  $G_error = $TRUE;

  my ($sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst) = localtime;
  my $date = sprintf("%02.02d:%02.02d:%02.02d", $hour, $min, $sec) ;
  COMMON::print_err("$date Error: $message\n") ;
}

#==============================================================================
#
#  CONFIGURATION FILE PROCESSING
#
#==============================================================================

#------------------------------------------------------------------------------
#
#  load cluster definition
#
#------------------------------------------------------------------------------

sub load_cluster
  {
    my ($filename) = @_ ;

    COMMON::print_debug($DEBUG_CONFIG, 
			"Loading cluster definition from $filename") ;

    if (! PARSER::parse_file(\@G_cluster_command, $filename)) {
      $G_error = $TRUE;
    }
  }

#------------------------------------------------------------------------------
#
#  load addon definition
#
#------------------------------------------------------------------------------

sub load_addon
  {
    my ($filename) = @_ ;

    my $i;

    if (! -f $filename) {
      $DO_ADDON = $FALSE ;
      return ;
    }

    $DO_ADDON = $TRUE;

    COMMON::print_debug($DEBUG_CONFIG,
			  "Loading Add-on definition from $filename") ;

    if (! PARSER::parse_file(\@G_addon_command, $filename)) {
      $G_error = $TRUE;
    }
 }

#------------------------------------------------------------------------------
#
#  check_alias
#
#  check if an interface is an alias of another and if this alias is 1
#
#------------------------------------------------------------------------------

sub check_alias {

  my ($node_id, $interface, $meaning, $nic_main, $nic_second) = @_ ;

  $nic_main =~  m/^([a-zA-Z]+[0-9]+)[:]?[0-9]*$/;
  my $phys_main = $1;

  $nic_second =~  m/^([a-zA-Z]+[0-9]+)[:]?([0-9]*)$/;
  my $phys_second = $1;
  my $alias_second = $2;

  if (($phys_main eq $phys_second) && ($alias_second eq "1")) {
    print_error("NODE $node_id: $meaning $nic_second cannot use 1 as an alias of the $interface interface");
    return $FALSE;
  }

  return $TRUE;
				
  
}

#------------------------------------------------------------------------------
#
#  check_public_interface
#
#  check if an interface is an alias of another
#
#     the second interface must be different from the main interface
#     or an alias of the main interface if the main is a physical one
#
#------------------------------------------------------------------------------

sub check_public_interface {

  my ($node_id, $interface, $meaning, $nic_main, $nic_second) = @_ ;

  $nic_main =~  m/^([a-zA-Z]+[0-9]+)[:]?[0-9]*$/;
  my $phys_main = $1;

  if ($nic_main eq $nic_second) {

    # it is exactly the same (both physical or alias)
    print_error("NODE $node_id: $meaning $nic_second already used for CGTP $interface");
    return $FALSE;

  } elsif ($phys_main eq $nic_second) {

    # if equal, public is physical and cgtp nic an alias
    print_error("NODE $node_id: $meaning $nic_second cannot be a physical interface supporting CGTP $interface as an alias");
    return $FALSE;

  }

}

#------------------------------------------------------------------------------
#
#  check cluster configuration
#
#------------------------------------------------------------------------------

sub check_cluster {

  my $i;
  my $j;
  my $nic_error = $FALSE;
  my $alias;

  my $nb_nodes = scalar(@NODE_LIST);
  
  # check mandatory information
  if ($CLUSTER eq "") {
    print_error("CLUSTER_ID not defined") ;
  }
  
  # check coherency and set default value
  
  # check that postfixes are all different
  if ($POST_NIC0 eq $POST_NIC1) {
    print_error("postfix for NIC0 and NIC1 must be different"); 
  }
  if ($POST_NIC0 eq $POST_CGTP) {
    print_error("postfix for NIC0 and CGTP must be different"); 
  }
  if ($POST_NIC1 eq $POST_CGTP) {
    print_error("postfix for NIC1 and CGTP must be different"); 
  }
  
  # check default network interfaces
  if ($USE_CGTP) {
    if (($MEN_NIC0 eq "") || ($MEN_NIC1 eq "")) {
      print_error("MEN_INTERFACES: both interfaces must be defined for MEN nodes") ;
      $nic_error = $TRUE;
    }	
    if (($nb_nodes > 2) && (($NMEN_NIC0 eq "") || ($NMEN_NIC1 eq ""))) {
      print_error("NMEN_INTERFACES: both interfaces must be defined for NMEN nodes") ;
      $nic_error = $TRUE;
    }	
    if ($USE_CLUSTER_NETWORK) {
      if ($CLUSTER_DOTTED_NIC1_NET eq "") {
	print_error("CLUSTER_NETWORK: NIC1 sub-network must be defined") ;
	$nic_error = $TRUE;
      }
      if ($CLUSTER_DOTTED_CGTP_NET eq "") {
	print_error("CLUSTER_NETWORK: CGTP sub-network must be defined") ;
	$nic_error = $TRUE;
      }
    }	
  } else {
    if ($MEN_NIC0 eq "") {
      print_error("MEN_INTERFACE: the first interface must be defined for MEN nodes") ;
      $nic_error = $TRUE;
    }	
    if (($nb_nodes > 2) && ($NMEN_NIC0 eq "")) {
      print_error("NMEN_INTERFACE: the first interface must be defined for NMEN nodes") ;
      $nic_error = $TRUE;
    }	
  }

  $MEN_NIC0 =~  m/^([a-zA-Z]+[0-9]+)[:]?([0-9]*)$/;
  $alias = $2;
  if (defined($alias) && ($alias eq "1")) {
    print_error("MEN_INTERFACES: $MEN_NIC0, alias 1 cannot be used for MEN nic0 (reserved for master IP address)");	
    $nic_error = $TRUE;
  }
  
  if ($USE_CGTP) {
    $MEN_NIC1 =~  m/^([a-zA-Z]+[0-9]+)[:]?([0-9]*)$/;
    $alias = $2;
    if (defined($alias) && ($alias eq "1")) {
      print_error("MEN_INTERFACES: $MEN_NIC1, alias 1 cannot be used for MEN nic1 (reserved for master IP address)");	
      $nic_error = $TRUE;
    }
  }
  
  # if there is a NIC error, exit because the following are not relevant
  if ($nic_error) {
    return ;
  }
  
  # check master eligible nodes
  # maybe there is only one node defined in case of error
  my $max = scalar(@NODE_LIST);
  if ($max > 2) {
    $max = 2;
  }
  for ($i = 0; $i < $max; $i++) {
    if ($NODE_LIST[$i]{nic0} eq "") {
      # no nic0 defined, set to the default value
      $NODE_LIST[$i]{nic0} = $MEN_NIC0;
    }
    if ($NODE_LIST[$i]{nic1} eq "") {
      # no nic1 defined, set to the default value
      $NODE_LIST[$i]{nic1} = $MEN_NIC1;
    }
  }
  
  # check public interfaces
  if ($USE_PUBLIC_NETWORK) {
    
    # public network defined
    
    for $i (0..$#NODE_LIST) {
      $NODE_LIST[$i]{pubConfigured} = $TRUE;
      
      if (($NODE_LIST[$i]{pubName} eq "") 
	  && ($NODE_LIST[$i]{pubIp} eq "") 
	  && ($NODE_LIST[$i]{pubNic} eq "")) {
	# public network not configured for this node
	$NODE_LIST[$i]{pubConfigured} = $FALSE;
	
      } else {
	
	# public network partially configured -> error
	if (($NODE_LIST[$i]{pubName} eq "")  || ($NODE_LIST[$i]{pubIp} eq "")  || ($NODE_LIST[$i]{pubNic} eq "")) {
	  print_error("Node id $NODE_LIST[$i]{id}, full definition for public network required (name, IP address and interface)");
	}
	
      }
    }
    
  } else {
    
    # public net work not defined

    for $i (0..$#NODE_LIST) {
	# public network not in use
	$NODE_LIST[$i]{pubConfigured} = $FALSE;
      if (($NODE_LIST[$i]{pubName} ne "") || ($NODE_LIST[$i]{pubIp} ne "") || ($NODE_LIST[$i]{pubNic} ne "")) {
	COMMON::warning("Node id $NODE_LIST[$i]{id}, attributes for the public network are ignored (PUBLIC_NETWORK not defined)");
      }
    }
  }
  
  # set the diskless indicator
  if ($DISKLESS_TYPE eq "STANDALONE") {
    $DISKLESS_CLUSTER=$FALSE;
  }
  
  # check that CGTP is used when diskless are standalone
  if ((! $USE_CGTP) && (! $DISKLESS_CLUSTER)) {
    print_error("CGTP is required when the diskless nodes are installed with CGTP standalone");
  }
  
  # DHCP configuration files can be local only if dynamic boot policy
  # is not used
  if (! $REPLICATED_DHCP_FILES && ($DISKLESS_BOOT_POLICY eq "DHCP_DYNAMIC")) {
    print_error("DHCP configuration on local partition not possible if dynamic boot policy is enable");
}

  # check diskless boot
  my $needed_arg = 0;
  my $arg_type = "";
  my $need_interface = $FALSE;
  if (($DISKLESS_BOOT_POLICY eq "DHCP_DYNAMIC") || 
      ($DISKLESS_BOOT_POLICY eq "NLS_DYNAMIC")) {
    $needed_arg = 0;
    $arg_type = "";
    # node-specific definition ignored as all diskless must be configured
    # identicallly
    $need_interface = $FALSE;
  } elsif (($DISKLESS_BOOT_POLICY eq "DHCP_STATIC") ||
	   ($DISKLESS_BOOT_POLICY eq "NLS_MAC_ADDR")) {
    $needed_arg = 2;
    $arg_type = "MAC addresses";
    $need_interface = $TRUE;
  } elsif ($DISKLESS_BOOT_POLICY eq "DHCP_CLIENT_ID") {
    $needed_arg = 1;
    $arg_type = "Client Id";
    $need_interface = $TRUE;
  } elsif ($DISKLESS_BOOT_POLICY eq "NLS_CPCI_SLOT") {
    print_error("Boot policy based on CPCI slots not yet supported by the installation tool");
    #      $needed_arg = 1;
    #      $arg_type = "";
    #      $need_interface = $TRUE;
  } elsif ($DISKLESS_BOOT_POLICY eq "NLS_SWITCH_PORT") {
    print_error("Boot policy based on switch ports not yet supported by the installation tool");
    #      $needed_arg = 2;
    #      $arg_type = "switch port numbers";
    #      $need_interface = $TRUE;
  }
  
  # check diskless nodes
  for $i (0..$#NODE_LIST) {

    if ($NODE_LIST[$i]{type} == $DATALESS_NODE) {
      
      if ($NODE_LIST[$i]{nic0} eq "") {
	# no nic0 defined, set to the default value
	$NODE_LIST[$i]{nic0} = $NMEN_NIC0;
      }
      if ($NODE_LIST[$i]{nic1} eq "") {
	# no nic1 defined, set to the default value
	$NODE_LIST[$i]{nic1} = $NMEN_NIC1;
      }
    } elsif ($NODE_LIST[$i]{type} == $DISKLESS_NODE) {
      
      if ($needed_arg == 2) {
	
	if (($NODE_LIST[$i]{eth0} eq "") || ($NODE_LIST[$i]{eth1} eq "")) {
	  print_error("NODE $NODE_LIST[$i]{id}: both $arg_type mandatory for the required boot policy ($DISKLESS_BOOT_POLICY)\n") ;
	}
	
      } elsif ($needed_arg == 1) {
	
	if ($NODE_LIST[$i]{eth0} eq "") {
	  print_error("NODE $NODE_LIST[$i]{id}: $arg_type mandatory for the required boot policy ($DISKLESS_BOOT_POLICY)\n") ;
	}
	
      } elsif ($needed_arg == 0) {
	
	if (($NODE_LIST[$i]{eth0} ne "") || ($NODE_LIST[$i]{eth1} ne "")) {
	  COMMON::warning("NODE $NODE_LIST[$i]{id}: Client ID ignored for the required boot policy ($DISKLESS_BOOT_POLICY)");
	}
	
      }
      
      if ($need_interface){
	if ($NODE_LIST[$i]{nic0} eq "") {
	  # no nic0 defined, set to the default value
	  $NODE_LIST[$i]{nic0} = $NMEN_NIC0;
	}
	if ($NODE_LIST[$i]{nic1} eq "") {
	  # no nic1 defined, set to the default value
	  $NODE_LIST[$i]{nic1} = $NMEN_NIC1;
	}
	
      } else {
	if (($NODE_LIST[$i]{nic0} ne "") || ($NODE_LIST[$i]{nic1} ne "")) {
	  COMMON::warning("NODE $NODE_LIST[$i]{id}: NIC0 and NIC1 ignored for the required policy ($DISKLESS_BOOT_POLICY)");
	}
	# set to the default value
	$NODE_LIST[$i]{nic0} = $NMEN_NIC0;
	$NODE_LIST[$i]{nic1} = $NMEN_NIC1;
      }
    }
  }

  # check all the network interfaces
  # - on master node, no interface (CGTP, external access or public network)
  #   can be an alias equal to 1 of a cgtp interface
  # - the public interface cannot be the physical interface of a CGTP
  #   defined as an alias
  for $i (0..$#NODE_LIST) {
    
    if ($NODE_LIST[$i]{type} == $MEN_NODE) {
      $NODE_LIST[$i]{nic0} =~  m/^([a-zA-Z]+[0-9]+)[:]?([0-9]*)$/;
      $alias = $2;
      if (defined($alias) && ($alias eq "1")) {
	print_error("NODE $NODE_LIST[$i]{id}: alias 1 cannot be used for nic0 (reserved for master IP address)");	
      }
      
      if ($USE_CGTP) {
	$NODE_LIST[$i]{nic1} =~  m/^([a-zA-Z]+[0-9]+)[:]?([0-9]*)$/;
	$alias = $2;
	if (defined($alias) && ($alias eq "1")) {
	  print_error("NODE $NODE_LIST[$i]{id}: alias 1 cannot be used for nic1 (reserved for master IP address)");	
	}
      }
    }
    
    if ($NODE_LIST[$i]{pubConfigured}) {
      
      check_public_interface(
			     $NODE_LIST[$i]{id},
			     "Nic0",
			     "public interface",
			     $NODE_LIST[$i]{nic0},
			     $NODE_LIST[$i]{pubNic}
			    );
      if ($NODE_LIST[$i]{type} == $MEN_NODE)  {
	check_alias (
		     $NODE_LIST[$i]{id},
		     "Nic0",
		     "public_interface",
		     $NODE_LIST[$i]{nic0},
		     $NODE_LIST[$i]{pubNic}
		    );
      }
      
      if ($USE_CGTP) {
	
	check_public_interface(
			       $NODE_LIST[$i]{id},
			       "Nic1",
			       "public interface",
			       $NODE_LIST[$i]{nic1},
			       $NODE_LIST[$i]{pubNic}
			      );
	if ($NODE_LIST[$i]{type} == $MEN_NODE)  {
	  check_alias (
		       $NODE_LIST[$i]{id},
		       "Nic1",
		       "public_interface",
		       $NODE_LIST[$i]{nic1},
		       $NODE_LIST[$i]{pubNic}
		      );
	}
	
      }
    }
    
    # only on the master node
    if (($NODE_LIST[$i]{type} == $MEN_NODE) && $USE_EXTERNAL_ACCESS) {
      
      check_alias (
		   $NODE_LIST[$i]{id},
		   "Nic0",
		   "external access",
		   $NODE_LIST[$i]{nic0},
		   $EXTERNAL_NIC
		  );
      
      if ($USE_CGTP) {
	check_alias(
		    $NODE_LIST[$i]{id},
		    "Nic1",
		    "public interface",
		    $NODE_LIST[$i]{nic1},
		    $EXTERNAL_NIC
		   );
      }
    }
  }
}
    
#------------------------------------------------------------------------------
#
#  load environment variables used for configuration
#
#------------------------------------------------------------------------------

sub load_variables {

  my $reason;
  my $solaris_install;

  # debug mode
  my $debug_mode;
  if (COMMON::var_defined("NHINSTALL_DEBUG", "", \$debug_mode)) {
    if ($debug_mode eq "all") {
      $DEBUG_FILE = $TRUE;
      $DEBUG_CMD  = $TRUE;
      $DEBUG_ACTION  = $TRUE;
      $DEBUG_CONFIG = $TRUE;
    } else {
      foreach my $mode (split(/,/, $debug_mode)) {
	if ($mode eq "action") {
	  $DEBUG_ACTION = $TRUE;
	} elsif ($mode eq "file") {
	  $DEBUG_FILE = $TRUE;
	} elsif ($mode eq "cmd") {
	  $DEBUG_CMD = $TRUE;
	} elsif ($mode eq "conf") {
	  $DEBUG_CONFIG = $TRUE;
	} else {
	  COMMON::warning("debug mode $mode unknown: ignored");
	}
      }
    }
  }
  
  # process variables
  if (! COMMON::var_defined("SERVER_INTERFACE", "", \$IFACE)) {
    print_error("SERVER_INTERFACE (network interface used to access cluster) is not defined") ;
  }

  if (! COMMON::var_defined("SERVER_NODE", "", \$SERVER_NODE)) {
    print_error("SERVER_NODE (node id of the installation server) is not defined") ;
  } else {
    if ($SERVER_NODE <= 0) {
      print_error("Wrong SERVER_NODE value ($SERVER_NODE): must be greater than 0") ;
    }
    if ($SERVER_NODE == $MASTER_ID) {
      print_error("Wrong SERVER_NODE value ($SERVER_NODE): cannot be equal to master node id") ;
    }
  }
  
  
  COMMON::var_defined("SOLARIS_INSTALL", "ALL", \$solaris_install);

  # all variables are intialized for SOLARIS_INSTALL=ALL
  if ($solaris_install eq "DISKLESS_ONLY") {

    $INSTALL_SOLARIS_ON_MEN = $FALSE;
    $INSTALL_SOLARIS_ON_DATALESS = $FALSE;

  } elsif ($solaris_install eq "DISKLESS_DATALESS_ONLY") {

    $INSTALL_SOLARIS_ON_MEN = $FALSE;

  } elsif ($solaris_install eq "NONE") {
    $INSTALL_SOLARIS_ON_MEN      = $FALSE;
    $INSTALL_SOLARIS_ON_DISKLESS = $FALSE;
    $INSTALL_SOLARIS_ON_DATALESS = $FALSE;
    $USE_SOLARIS_DISTRIB         = $FALSE;

  } elsif ($solaris_install ne "ALL") {
    print_error("SOLARIS_INSTALL: invalid value ($solaris_install)") ;
  }
  
  COMMON::var_defined("AUTO_REBOOT", "YES", \$AUTO_REBOOT) ;
  if (! PARSER::is_boolean(\$AUTO_REBOOT, $PARSER::CMD_BOOLEAN, \$reason)) {
    print_error("AUTO_REBOOT: $reason") ;
  }
  
  if (! COMMON::var_defined("SOLARIS_DIR", "", \$MEN_SOLARIS_DIR)) {
    print_error("variable SOLARIS_DIR not defined") ;
  }
  if (! COMMON::var_defined("NHAS2_PRODUCT_DIR", "", \$NHAS_DIR)) {
    print_error("variable NHAS2_PRODUCT_DIR not defined") ;
  }
  if (! COMMON::var_defined("WORKING_DIR", "", \$WORKING_DIR)) {
    print_error("variable WORKING_DIR not defined") ;
  }

  if (! COMMON::var_defined("DISKLESS_SOLARIS_DIR", "", 
			    \$DISKLESS_SOLARIS_DIR)) {
    $DISKLESS_SOLARIS_DIR = $MEN_SOLARIS_DIR
  }
  if (! COMMON::var_defined("DATALESS_SOLARIS_DIR", "", 
			    \$DATALESS_SOLARIS_DIR)) {
    $DATALESS_SOLARIS_DIR = $MEN_SOLARIS_DIR
  }
  COMMON::var_defined("NHINSTALL_TESTING", "", \$TESTING);
}

#------------------------------------------------------------------------------
#
#  look for a node based on its node id. Returns a boolean
#
#------------------------------------------------------------------------------

sub node_search {
  
  my ($node_id, $ref_idx) = @_;

  for my $i (0..$#NODE_LIST) {
    if ($NODE_LIST[$i]{id} == $node_id) {
      $$ref_idx = $i;
      return $TRUE;
    }
  }

  return $FALSE;
}

#------------------------------------------------------------------------------
#
# control the coherency node type <-> slice type
#
#------------------------------------------------------------------------------

sub is_link_coherent {

  my ($slice_idx, $node_idx) = @_;

  my $node_type = $NODE_LIST[$node_idx]{node_type} ;

  if ($node_type == $DISKLESS_NODE) {
    print_error("Slice $SLICE_LIST[$slice_idx]{name}: cannot associated to a diskless node ($NODE_LIST[$node_idx]{id})");
    return $FALSE;
  }

  if ($SLICE_LIST[$slice_idx]{bitmap} ne "") {
    if ($node_type != $MEN_NODE) {
      print_error("Slice $SLICE_LIST[$slice_idx]{name}: a replicated slice must be defined only for MEN ($NODE_LIST[$node_idx]{id} not a MEN)");
      return $FALSE;
    }
  }

  return $TRUE;
}

#------------------------------------------------------------------------------
#
#  for each slice:
#  - check if the link is possible:
#    . node not a diskless
#    . replicated slice only on MEN
#  - link it to the concerned nodes
#
#------------------------------------------------------------------------------

sub check_and_link_slice_to_nodes {

  for my $i (0..$#SLICE_LIST) {

    my $node_type;
    my $node_idx;

    # convert the scope in uppercase
    $SLICE_LIST[$i]{scope} =~ tr/a-z/A-Z/;

    # process each value of the scope
    foreach my $value (split(/,/,$SLICE_LIST[$i]{scope})) {

      if ($value =~ m/^[0-9]*$/) {
	# numeric value: node id

	# check if the node is a MEN or a dataless
	if (! node_search($value, \$node_idx)) {
	  # the node does not exist
	  print_error("slice $SLICE_LIST[$i]{name}: node $value not defined");
	} else {
	  # the node is not a MEN or a DATALESS
	  if (($NODE_LIST[$node_idx]{type} != $MEN_NODE) 
	      && ($NODE_LIST[$node_idx]{type} != $DATALESS_NODE)) {
	    print_error("slice $SLICE_LIST[$i]{name}: node $value must be a MEN or a dataless");
	  } else {

	    # that's OK: we add the slice in the slice list of the node
	    push @{$NODE_LIST[$node_idx]{slice}}, $i;
	  }
	}
      
	
      } else {

	# alphanumeric value: node type
	my $ok = $TRUE;
	if ($value eq "MEN") {
	  $node_type = $MEN_NODE;
	} elsif ($value eq "DATALESS") {
	  $node_type = $DATALESS_NODE;
	} else {
	  print_error("slice $SLICE_LIST[$i]{name}: invalid node type ($value)");
	  $ok = $FALSE;
	}
	if ($ok) {
	  for $node_idx (0..$#NODE_LIST) {
	    if ($NODE_LIST[$node_idx]{type} == $node_type) {
	      push @{$NODE_LIST[$node_idx]{slice}}, $i;
	    }
	  }
	}
      }
    }
  }

}

#------------------------------------------------------------------------------
#
#  check slice configuration for MEN
#
#------------------------------------------------------------------------------

sub check_men_slice {

  my $idx ;



  # check for each MEN
  for (my $node = 0 ; $node < scalar(@NODE_LIST) ; $node++) {

    if ($NODE_LIST[$node]{type} != $MEN_NODE) {
      next;
    }
    
    my $node_id = $NODE_LIST[$node]{id};
    my $nodename = COMMON::build_node_name($node_id, "");
    
    # check that mandatory slices are in the scope of the current node

    if (! COMMON::slice_search($node_id, "mounting_point",
			       $SLICE_ROOT, \$idx)) {
      print_error("Node $node_id: a slice with $SLICE_ROOT (root) as mounting point is required") ;
    } else {
      if ($SLICE_LIST[$idx]{size} < $SLICE_ROOT_MIN_SIZE) {
	print_error("Node $node_id: minimum size for $SLICE_ROOT (root) must be $SLICE_ROOT_MIN_SIZE") ;
      }
    }
    
    if (! COMMON::slice_search($node_id, "mounting_point",
		       $SLICE_SUNWCGHA, \$idx)) {
      print_error("Node $node_id: a slice with $SLICE_SUNWCGHA as mounting point is required") ;
    } else {
      if ($SLICE_LIST[$idx]{size} < $SLICE_SUNWCGHA_MIN_SIZE) {
	print_error("Node $node_id: minimum size for $SLICE_SUNWCGHA must be $SLICE_SUNWCGHA_MIN_SIZE") ;
      }
    }
    
    # if the user ask for no diskless, SLICE_EXPORT is not mandatory
    my $diskless_required = SEQUENCER::get_data("DISKLESS_ENV");
    if ((scalar(@NODE_LIST) > 2) || ($diskless_required)) {
      if (! COMMON::slice_search($node_id, "mounting_point", 
				 $SLICE_EXPORT, \$idx)) {
	print_error ("Node $node_id: a slice with $SLICE_EXPORT as mounting point is required") ;
      } else {
	my $MIN_SIZE = $SLICE_EXPORT_MIN_SIZE + $DISKLESS_COUNT * $SLICE_EXPORT_MIN_SIZE_PER_DISKLESS;
	if ($SLICE_LIST[$idx]{size} < $MIN_SIZE) {
	  COMMON::warning("Node $node_id: minimum size for $SLICE_EXPORT should be $MIN_SIZE") ;
	}
      }
    }
  }
}

#------------------------------------------------------------------------------
#
#  check slice configuration for DATALESS
#
#------------------------------------------------------------------------------

sub check_dataless_slice {

  my $idx ;



  # check for each dataless nodes
  for (my $node = 0 ; $node < scalar(@NODE_LIST) ; $node++) {

    if ($NODE_LIST[$node]{type} != $DATALESS_NODE) {
      next;
    }
    
    my $node_id = $NODE_LIST[$node]{id};
    my $nodename = COMMON::build_node_name($node_id, "");
    
    # check that mandatory slices are in the scope of the current node

    if (! COMMON::slice_search($node_id, "mounting_point",
			       $SLICE_ROOT, \$idx)) {
      print_error("Node $node_id: a slice with $SLICE_ROOT (root) as mounting point is required") ;
    }
    else {
      if ($SLICE_LIST[$idx]{size} < $SLICE_ROOT_MIN_SIZE) {
	print_error("Node $node_id: minimum size for $SLICE_ROOT (root) must be $SLICE_ROOT_MIN_SIZE") ;
      }
    }

    foreach my $local_slice (@{$NODE_LIST[$node]{slice}}) {
      # check that there is neither replicated nor bitmap slices
      if ($SLICE_LIST[$local_slice]{bitmap} ne "") {
	print_error("Node $node_id: replicated slice $SLICE_LIST[$local_slice]{name} not allowed for a dataless node");
      }
      if ($SLICE_LIST[$local_slice]{mounting_point} eq $SLICE_UNMOUNTED) {
	print_error("Node $node_id: bitmap slice $SLICE_LIST[$local_slice]{name} not allowed for a dataless node");
      }
    }
    
  }
}
  
#------------------------------------------------------------------------------
#
#  check replicated slices
#
#------------------------------------------------------------------------------

sub check_replicated_slice {

  my ($current_node_id, $other_node_id) = @_;

  my $local_slice;
  my $local_bitmap;
  my $remote_slice;
  my $remote_bitmap;

  # for each replicated slice
  # . check if the equivalent slice and bitmap exist on the other MEN
  #   with the same size and the same name (physical or metadevice)

  my $node_idx = COMMON::getnode($current_node_id);
  for my $local_slice (@{$NODE_LIST[$node_idx]{slice}}) {
      
    if ($SLICE_LIST[$local_slice]{bitmap} ne "") {
      
      # check if the local bitmap exists
      if (! COMMON::slice_search($current_node_id, "name", 
				 $SLICE_LIST[$local_slice]{bitmap}, 
				 \$local_bitmap)) {
	print_error("Node $current_node_id: bitmap slice $SLICE_LIST[$local_slice]{bitmap} for slice $SLICE_LIST[$local_slice]{name} not defined");
	next ;
      }
      
      # the slice can't replicate itself
      if ($local_slice == $local_bitmap) {
	print_error("Node $current_node_id: slice $SLICE_LIST[$local_slice]{name} can not be replicated on itself");
	next;
      }
      
      # the bitmap slice is defined as "unnamed"
      if ($SLICE_LIST[$local_bitmap]{mounting_point} ne $SLICE_UNMOUNTED) {
	print_error("Node $current_node_id: bitmap slice $SLICE_LIST[$local_slice]{bitmap} for slice $SLICE_LIST[$local_slice]{name} is not an $SLICE_UNMOUNTED slice");
	next ;
      }
      
      # the bitmap is not used for another replicated slice on the same node
      for my $j (@{$NODE_LIST[$node_idx]{slice}}) {
	if (($local_slice ne $j) && ($SLICE_LIST[$j]{bitmap} ne "")) {
	  if ($SLICE_LIST[$local_slice]{bitmap}
	      eq $SLICE_LIST[$j]{bitmap}) {
	    print_error("Node $current_node_id: bitmap slice $SLICE_LIST[$local_slice]{bitmap} is used twice for different slices");
	  }
	}
      }
      
      # check if the remote slice exists
      if (! COMMON::slice_search($other_node_id, "mounting_point", 
				 $SLICE_LIST[$local_slice]{mounting_point}, 
				 \$remote_slice)) {
	print_error("Node $current_node_id: slice $SLICE_LIST[$local_slice]{name} as no corresponding remote slice") ;
	next ;
      }
      
      # check if the remote slice has the same name
      if (! $USE_SVM && 
	  ($SLICE_LIST[$local_slice]{raw} 
	   ne $SLICE_LIST[$remote_slice]{raw})) {
	print_error("Node $current_node_id: slice $SLICE_LIST[$local_slice]{name} has not the same name of its remote slice (use LOGICAL_SLICE_SUPPORT if required)") ;
	next;
      }

      # check if the remote slice has the same size
      if ($SLICE_LIST[$local_slice]{size} 
	  != $SLICE_LIST[$remote_slice]{size}) {
	print_error("Node $current_node_id: slice $SLICE_LIST[$local_slice]{name} has not the same size of its remote slice") ;
	next;
      }
      
      # check if the remote slice is replicated
      if ($SLICE_LIST[$remote_slice]{bitmap} eq "") {
	print_error("Node $current_node_id: remote slice with $SLICE_LIST[$local_slice]{mounting_point} as mounting point not replicated") ;
	next;
      }
      
      # check if the remote bitmap exists
      if (! COMMON::slice_search($other_node_id, "name", 
				 $SLICE_LIST[$remote_slice]{bitmap}, 
				 \$remote_bitmap)) {
	print_error("Node $current_node_id: remote bitmap slice $SLICE_LIST[$remote_slice]{bitmap} for slice $SLICE_LIST[$local_slice]{name} not defined");
	next ;
      }
      
      # check if the remote bitmap has the same name
      if (! $USE_SVM
	  && ($SLICE_LIST[$local_bitmap]{raw} 
	      ne $SLICE_LIST[$remote_bitmap]{raw})) {
	print_error("Node $current_node_id: bitmap for slice $SLICE_LIST[$local_slice]{name} has not the same name of its remote slice (use LOGICAL_SLICE_SUPPORT if required)") ;
	next;
      }

      # check if the remote bitmap has the same size
      if ($SLICE_LIST[$local_bitmap]{size} 
	  != $SLICE_LIST[$remote_bitmap]{size}) {
	print_error("Node $current_node_id: bitmap for slice $SLICE_LIST[$local_slice]{name} has not the same size of its remote slice.") ;
	next;
      }
      
      # all info references for the replicated slices
      $SLICE_LIST[$local_slice]{local_bitmap_idx} = $local_bitmap;
      $SLICE_LIST[$local_slice]{remote_slice_idx} = $remote_slice;
      $SLICE_LIST[$local_slice]{remote_bitmap_idx} = $remote_bitmap;
      # each bitmap references its equivalent on the otehr nodes
      $SLICE_LIST[$remote_bitmap]{equivalent} = $local_bitmap;
      $SLICE_LIST[$local_bitmap]{equivalent} = $remote_bitmap;
      
    } # if bitmap
  } # for
} # sub

#------------------------------------------------------------------------------
#
#  check exported directories
#
#------------------------------------------------------------------------------

sub check_exported {

  my $i;
  my $j;
  my $found;
  my $idx;

  #
  # each exported directory must be a child of a replicated partition
  # (no need to check for each MEN: existence of equivalent mounting point
  # as already been checked)
  #
  for $i (0..$#DIR_EXPORTED) {
    $found = $FALSE ;
    for $j (0..$#SLICE_LIST) {
      if ($SLICE_LIST[$j]{bitmap} ne "") {
	# look for slice name as a subtring of directory name
	# add a slash at the end of the exported dir and mounting point name
	# if not already present to avoid problem due to similar partial name
	# ex: exported=/SUNWcgha/local/foodoo and mounting point /SUNWcgha/local/foo
	my $exported = $DIR_EXPORTED[$i] ;
	if (substr($exported, length($exported)-1, 1) ne "/") {
	  $exported = $exported . "/" ;
	}
	my $mounting_point = $SLICE_LIST[$j]{mounting_point} ;
	if (substr($mounting_point, length($mounting_point)-1, 1) ne "/") {
	  $mounting_point = $mounting_point . "/" ;
	}
	
	my $lg = length($mounting_point) ;
	if (substr($exported, 0, $lg) eq $mounting_point) {
	  $found = $TRUE ;
	  last ;
	}
      }
    }
    if (! $found) {
      print_error("Exported directory $DIR_EXPORTED[$i] is not located on a replicated slice") ;
    }
  }
  
  #
  # check that an exported directory is not the child of an already exported directory
  #
  for $i (0..$#DIR_EXPORTED) {
    $found = $FALSE ;
    for $j (0..$#DIR_EXPORTED) {
      if ($i == $j) {
	next ;
      }
      
      my $exported1 = $DIR_EXPORTED[$i] ;
      if (substr($exported1, length($exported1)-1, 1) ne "/") {
	$exported1 = $exported1 . "/" ; 
      }
      my $exported2 = $DIR_EXPORTED[$j] ;
      if (substr($exported2, length($exported2)-1, 1) ne "/") {
	$exported2 = $exported2 . "/" ;
      }
      
      my $lg = length($exported1) ;
      if (substr($exported2, 0, $lg) eq $exported1) {
	$found = $TRUE ;
	$idx = $j; # problem with scope of variable
	last ;
      }
    }

    if ($found) {
      print_error("Exported directory $DIR_EXPORTED[$idx] is child of an already exported directory ($DIR_EXPORTED[$i])") ;
    }
  }
}

#------------------------------------------------------------------------------
#
#  check remote mounted directories
#
#------------------------------------------------------------------------------

sub check_remote {

  my $i;
  my $j;
  
  #
  # each remote mounted directory must be a child of an exported directory
  # (no need to check for each MEN: existence of equivalent mounting point
  # as already been checked)
  #
  for $i (0..$#DIR_MOUNTED) {
    
    # set the value of the "noac" option at the same time
    # if not defined (added by the user).
    if (! defined($DIR_MOUNTED[$i]{noac})) {
      $DIR_MOUNTED[$i]{noac} = $RNFS_NOAC;
    }
    
    my $found = $FALSE ;
    for $j (0..$#DIR_EXPORTED) {
      
      # look for exported directory name as a subtring of remote directory name
      # add a slash at the end of the exported dir and remote directory
      # if not already present to avoid problem due to similar partial name
      # ex: exported=/user1 and mounting point /user12
      my $remote = $DIR_MOUNTED[$i]{remote} ;
      if (substr($remote, length($remote)-1, 1) ne "/") {
	$remote = $remote . "/" ;
      }
      my $exported = $DIR_EXPORTED[$j] ;
      if (substr($exported, length($exported)-1, 1) ne "/") {
	$exported = $exported . "/" ;
      }
      
      my $lg = length($exported);
      if (substr($remote, 0, $lg) eq $exported) {
	$found = $TRUE ;
	last ;
      }
    }
    if (! $found) {
      print_error("Remote directory $DIR_MOUNTED[$i]{remote} for mounting point $DIR_MOUNTED[$i]{local} is not child of an exported directory") ;
    }
  }
}

#------------------------------------------------------------------------------
#
#  check slices when SVM is selected
#
#------------------------------------------------------------------------------

sub check_svm {

  my $idx ;

  # no replica
  # - on MEN if LOGICAL_SLICE_SUPPORT is not set
  # - on dataless

  for (my $node = 0 ; $node < scalar(@NODE_LIST) ; $node++) {
    
    if ((($NODE_LIST[$node]{type} == $MEN_NODE)
	 && (! $USE_SVM))
	|| ($NODE_LIST[$node]{type} == $DATALESS_NODE)
       ) {
      my $node_id = $NODE_LIST[$node]{id};
      if (COMMON::slice_search($node_id, "mounting_point", 
			       $SLICE_TAG_REPLICA, \$idx)) {
	if ($NODE_LIST[$node]{type} == $MEN_NODE) {
	  print_error("Node $node_id: a slice with $SLICE_TAG_REPLICA attribute (as mounting point) only allowed with LOGICAL_SLICE_SUPPORT")
	} else {
	  print_error("Node $node_id: a slice with $SLICE_TAG_REPLICA attribute (as mounting point) not allowed on dataless node")
	}
      }
    }
  }
    
  if (! $USE_SVM) {
    return ;
  }

  if (! $INSTALL_SOLARIS_ON_MEN) {
    print_error("LOGICAL_SLICE_SUPPORT must be set to NO when Solaris installation is skipped on master-eligible nodes (SVM configuration is under your responsibility)");
  }

  for (my $node = 0 ; $node < scalar(@NODE_LIST) ; $node++) {

    if ($NODE_LIST[$node]{type} != $MEN_NODE) {
      next ;
    }

    my $node_id = $NODE_LIST[$node]{id};

    # for each slice replicated or for storing bitmap slice
    # a concat metadevice will be created
    # raw device and block device are changed accordingly
    my $meta_number = 0;
    foreach my $i (@{$NODE_LIST[$node]{slice}}) {
      if (($SLICE_LIST[$i]{bitmap} ne "")
	  || ($SLICE_LIST[$i]{mounting_point} eq $SLICE_UNMOUNTED)) {

	# for MEN1, a device number is assigned and the corresponding names
	# are created.
	# for MEN2, the same name must be used (required fro NFS)

	if ($node_id == $MEN1_ID) {
	  $meta_number++;
	  $SLICE_LIST[$i]{metadevice} = "d$meta_number";
	  $SLICE_LIST[$i]{raw} = "/dev/md/rdsk/d$meta_number";
	  $SLICE_LIST[$i]{block} = "/dev/md/dsk/d$meta_number";
	} else {
	  if ($SLICE_LIST[$i]{bitmap} ne "") {
	    # it's a replicated slice
	    $idx = $SLICE_LIST[$i]{remote_slice_idx};
	  } else {
	    # it's a bitmap slice
	    $idx = $SLICE_LIST[$i]{equivalent};
	  }
	  $SLICE_LIST[$i]{metadevice} = $SLICE_LIST[$idx]{metadevice} ;
	  $SLICE_LIST[$i]{raw} =  $SLICE_LIST[$idx]{raw} ;
	  $SLICE_LIST[$i]{block} =$SLICE_LIST[$idx]{block} ;
	}
      }
    }
    
    # check slice used for meta database
    if (! COMMON::slice_search($node_id, "mounting_point", 
			       $SLICE_TAG_REPLICA, \$idx)) {
      print_error("Node $node_id: a slice with $SLICE_TAG_REPLICA attribute (as mounting point) is required") ;
    } else {
      # its name is changed (only unnamed is supported by jumpstart)
      $SLICE_LIST[$idx]{metadb} = $TRUE;
    }
  }
}

#------------------------------------------------------------------------------
#
#  display_conf
#
#    display loaded configuration when debug is requested
#
#------------------------------------------------------------------------------

sub display_conf {


  my $network;
  my $netmask;
  if (! $DEBUG_CONFIG) {
    return ;
  }

  # display environment
  COMMON::print_debug($DEBUG_CONFIG, "Environment");
  COMMON::print_debug($DEBUG_CONFIG, "---------------");
  COMMON::print_debug($DEBUG_CONFIG, "    Solaris intallation:");
  COMMON::print_debug($DEBUG_CONFIG, "    . on MEN:            $INSTALL_SOLARIS_ON_MEN");
  COMMON::print_debug($DEBUG_CONFIG, "    . on diskless:       $INSTALL_SOLARIS_ON_DISKLESS");
  COMMON::print_debug($DEBUG_CONFIG, "    . access to dist:    $USE_SOLARIS_DISTRIB");
  COMMON::print_debug($DEBUG_CONFIG, "    Auto-reboot:         $AUTO_REBOOT");
  COMMON::print_debug($DEBUG_CONFIG, "    patch smosservice:   $PATCH_WITH_SMOSSERVICE");
  COMMON::print_debug($DEBUG_CONFIG, "");
  
  # display general information
  COMMON::print_debug($DEBUG_CONFIG, "General setting");
  COMMON::print_debug($DEBUG_CONFIG, "---------------");

  COMMON::print_debug($DEBUG_CONFIG, "    Cluster Id:          $CLUSTER");
  COMMON::print_debug($DEBUG_CONFIG, "    Arch:                $ARCH");
  COMMON::print_debug($DEBUG_CONFIG, "    Nic0 postfix:        $POST_NIC0");
  COMMON::print_debug($DEBUG_CONFIG, "    Nic1 postfix:        $POST_NIC1");
  COMMON::print_debug($DEBUG_CONFIG, "    CGTP postfix:        $POST_CGTP");
  COMMON::print_debug($DEBUG_CONFIG, "    Boot policy:         $DISKLESS_BOOT_POLICY");
  COMMON::print_debug($DEBUG_CONFIG, "    CGTP used:           $USE_CGTP");
  COMMON::print_debug($DEBUG_CONFIG, "    SVM used:            $USE_SVM");
  COMMON::print_debug($DEBUG_CONFIG, "    DCSS used:           $USE_DCSS");
  COMMON::print_debug($DEBUG_CONFIG, "    WDT used:            $USE_WDT");
  COMMON::print_debug($DEBUG_CONFIG, "    Diskless in cluster: $DISKLESS_CLUSTER");

  COMMON::print_debug($DEBUG_CONFIG, "    Netmask:             $CLUSTER_DOTTED_NETMASK");
  COMMON::print_debug($DEBUG_CONFIG, "    NIC0 subnet:         $CLUSTER_DOTTED_NIC0_NET");
  if ($USE_CGTP) {
    COMMON::print_debug($DEBUG_CONFIG, "    NIC1 subnet:         $CLUSTER_DOTTED_NIC1_NET");
    COMMON::print_debug($DEBUG_CONFIG, "    CGTP subnet:         $CLUSTER_DOTTED_CGTP_NET");
  }
  COMMON::print_debug($DEBUG_CONFIG, "");
  
  # display server information
  COMMON::print_debug($DEBUG_CONFIG, "Installation server");
  COMMON::print_debug($DEBUG_CONFIG, "-------------------");
  COMMON::print_debug($DEBUG_CONFIG, "");
  COMMON::print_debug($DEBUG_CONFIG, "    Node Id:           $SERVER_NODE");
  COMMON::print_debug($DEBUG_CONFIG, "    IP:                $SERVER_IP");
  COMMON::print_debug($DEBUG_CONFIG, "    Network interface: $IFACE");
  COMMON::print_debug($DEBUG_CONFIG, "    Network:           $DOTTED_NETWORK");
  COMMON::print_debug($DEBUG_CONFIG, "    Netmask:           $DOTTED_NETMASK");
  COMMON::print_debug($DEBUG_CONFIG, "");

  # display the node list
  COMMON::print_debug($DEBUG_CONFIG, "Node list");
  COMMON::print_debug($DEBUG_CONFIG, "---------");
  
 
  for my $node (0..$#NODE_LIST) {
    my $node_id  = $NODE_LIST[$node]{id};
    my $nodename = COMMON::build_node_name($node_id, "");
    COMMON::print_debug($DEBUG_CONFIG, "");
    COMMON::print_debug($DEBUG_CONFIG, 
			"  Node ID: $NODE_LIST[$node]{id}");
    COMMON::print_debug($DEBUG_CONFIG,
			"    name:              $nodename");
    COMMON::print_debug($DEBUG_CONFIG,
			"    MAC address #0:    $NODE_LIST[$node]{eth0}");
    COMMON::print_debug($DEBUG_CONFIG, 
			"    MAC address #1:    $NODE_LIST[$node]{eth1}");
    COMMON::print_debug($DEBUG_CONFIG,
			"    net interface #0:  $NODE_LIST[$node]{nic0}");
    COMMON::print_debug($DEBUG_CONFIG, 
			"    net interface #1:  $NODE_LIST[$node]{nic1}");
    COMMON::print_debug($DEBUG_CONFIG,
			"    IP #0:             $NODE_LIST[$node]{nic0Ip}");
    if ($USE_CGTP) {
      COMMON::print_debug($DEBUG_CONFIG,
			  "    IP #1:             $NODE_LIST[$node]{nic1Ip}");
      COMMON::print_debug($DEBUG_CONFIG,
			  "    IP CGTP:           $NODE_LIST[$node]{cgtpIp}");
    }
    if ($NODE_LIST[$node]{pubConfigured}) {
      COMMON::print_debug($DEBUG_CONFIG, 
			  "    public host name:  $NODE_LIST[$node]{pubName}");
      COMMON::print_debug($DEBUG_CONFIG, 
			  "    public interface:  $NODE_LIST[$node]{pubNic}");
      COMMON::print_debug($DEBUG_CONFIG, 
			  "    public IP address: $NODE_LIST[$node]{pubIp}");
    }

    COMMON::print_debug($DEBUG_CONFIG, "");
    
    if ($NODE_LIST[$node]{type} == $DISKLESS_NODE) {
      next;
    }

    # display the slice list for the node
    COMMON::print_debug($DEBUG_CONFIG, "    Slice list");
    
    if ($NODE_LIST[$node]{type} == $DISKLESS_NODE) {
      next;
    }
      
    for my $slice (@{$NODE_LIST[$node]{slice}}) {
      
      COMMON::print_debug($DEBUG_CONFIG, "");
      COMMON::print_debug($DEBUG_CONFIG,
			  "      Name: $SLICE_LIST[$slice]{name}");
      COMMON::print_debug($DEBUG_CONFIG,
			  "        mounting point: $SLICE_LIST[$slice]{mounting_point}");
      COMMON::print_debug($DEBUG_CONFIG,
			  "        size:           $SLICE_LIST[$slice]{size}");
      COMMON::print_debug($DEBUG_CONFIG,
			  "        option:         $SLICE_LIST[$slice]{option}");
      COMMON::print_debug($DEBUG_CONFIG,
			  "        raw device:     $SLICE_LIST[$slice]{raw}");
      COMMON::print_debug($DEBUG_CONFIG,
			  "        block device:   $SLICE_LIST[$slice]{block}");
      COMMON::print_debug($DEBUG_CONFIG,
			  "        metadevice:     $SLICE_LIST[$slice]{metadevice}");
      COMMON::print_debug($DEBUG_CONFIG,
			  "        metadb:         $SLICE_LIST[$slice]{metadb}");
    }
  }

  COMMON::print_debug($DEBUG_CONFIG, "");
    
  # display the list of exported directories
  COMMON::print_debug($DEBUG_CONFIG, "Exported directories");
  COMMON::print_debug($DEBUG_CONFIG, "--------------------");
  COMMON::print_debug($DEBUG_CONFIG, "");
  for my $i (0..$#DIR_EXPORTED) {
    COMMON::print_debug($DEBUG_CONFIG, "    $DIR_EXPORTED[$i]");
  }

  COMMON::print_debug($DEBUG_CONFIG, "");

  # display the list of mounted directories
  COMMON::print_debug($DEBUG_CONFIG, "Mounted directories");
  COMMON::print_debug($DEBUG_CONFIG, "-------------------");
  for my $i (0..$#DIR_MOUNTED) {
    COMMON::print_debug($DEBUG_CONFIG, "");
    COMMON::print_debug($DEBUG_CONFIG, "    local:  $DIR_MOUNTED[$i]{local}");
    COMMON::print_debug($DEBUG_CONFIG, "    remote: $DIR_MOUNTED[$i]{remote}");
    COMMON::print_debug($DEBUG_CONFIG, "    noac:   $DIR_MOUNTED[$i]{noac}");
  }

  COMMON::print_debug($DEBUG_CONFIG, "");

  # display the vendor list
  if (scalar(@VENDOR_TYPE) > 0) {
    COMMON::print_debug($DEBUG_CONFIG, "Vendor type");
    COMMON::print_debug($DEBUG_CONFIG, "-----------");
    COMMON::print_debug($DEBUG_CONFIG, "");
    for my $i (0..$#VENDOR_TYPE) {
      COMMON::print_debug($DEBUG_CONFIG, "    $VENDOR_TYPE[$i]");
    }
  }
  

  # display the patch list
  if (scalar(@ADDON_LIST) > 0) {
    COMMON::print_debug($DEBUG_CONFIG, "");
    COMMON::print_debug($DEBUG_CONFIG, "Addon list");
    COMMON::print_debug($DEBUG_CONFIG, "-----------");
    for my $i (0..$#ADDON_LIST) {
      COMMON::print_debug($DEBUG_CONFIG, "");
      COMMON::print_debug($DEBUG_CONFIG, "  $ADDON_LIST[$i]{type}: $ADDON_LIST[$i]{reference}");
      COMMON::print_debug($DEBUG_CONFIG, "    Directory:     $ADDON_LIST[$i]{dir}");
      COMMON::print_debug($DEBUG_CONFIG, "    Sub-directory: $ADDON_LIST[$i]{subdir}");
      COMMON::print_debug($DEBUG_CONFIG, "    Phase:         $ADDON_LIST[$i]{phase}");
      COMMON::print_debug($DEBUG_CONFIG, "    Scope:         $ADDON_LIST[$i]{scope}");
      COMMON::print_debug($DEBUG_CONFIG, "    On MEN:        $ADDON_LIST[$i]{men}");
      COMMON::print_debug($DEBUG_CONFIG, "    On DISKLESS:   $ADDON_LIST[$i]{diskless}");
      COMMON::print_debug($DEBUG_CONFIG, "    On DATALESS:   $ADDON_LIST[$i]{dataless}");

      if ($ADDON_LIST[$i]{type} == $ADDON_PATCH) {

	my $method = "Default";
	if ($ADDON_LIST[$i]{method} == $STANDARD_METHOD) {
	  $method = "patchadd";
	} elsif ($ADDON_LIST[$i]{method} == $SMOSSERVICE_METHOD) {
	  $method = "smosservice";
	} elsif ($ADDON_LIST[$i]{method} == $PATCH_WITH_PKGADD_METHOD) {
	$method = "patch with pkgadd";
      } else {
	$method = "UNKNOWN";
      }
	COMMON::print_debug($DEBUG_CONFIG, "    Method:        $method");  
      }
    }
  }
  
  COMMON::print_debug($DEBUG_CONFIG, "");
}

#------------------------------------------------------------------------------
#
#  checking of the configuration
#
#------------------------------------------------------------------------------

sub common_check
  {
    
    if (scalar(@NODE_LIST) < 2) {
      print_error("At least 2 nodes must be defined (2 master eligible nodes)") ;
    }

# no more tested.
#------
#    my $nb = scalar(@NODE_LIST);
#
#    if ($nb > $MAX_NODES) {
#      COMMON::warning("$nb nodes are configured for installation");
#      COMMON::warning("but only clusters up to $MAX_NODES nodes are supported (including 2 master eligible nodes)") ;
#    }
#------

    check_cluster ;
    check_and_link_slice_to_nodes();
    check_men_slice ;
    check_replicated_slice($MEN1_ID, $MEN2_ID);
    check_replicated_slice($MEN2_ID, $MEN1_ID);
    check_dataless_slice ;
    check_exported ;
    check_remote ;
    check_svm;
  }

#------------------------------------------------------------------------------
#
#  check that a Host ID is compatible with a subnetwork mask
#
#------------------------------------------------------------------------------

sub checkHostId {
  my ($hostId, $bitmask) = @_;

  if (($hostId & $bitmask) != 0) {
    return $FALSE;
  }

  return $TRUE;
}

#------------------------------------------------------------------------------
#
#  check that an IP address is compatible with a subnetwork mask
#
#------------------------------------------------------------------------------

sub checkIp {
  my ($bitip, $bitmask, $bitsubnet) = @_;

  if (($bitip & $bitmask) != ($bitsubnet & $bitmask)) {
    return $FALSE;
  }

  return $TRUE;
}
  
#------------------------------------------------------------------------------
#
#  configure global variables that are deducted from the loaded info
#  used by check_conf.pl
#
#------------------------------------------------------------------------------

sub configure_global {
  
  # encrypt the password
  $ENCRYPTED_PASSWORD = crypt ($PASSWORD, "NH");

  # set the default cluster network characteristics
  if (! $USE_CLUSTER_NETWORK) {
    $CLUSTER_DOTTED_NETMASK  = "255.255.255.0";
    $CLUSTER_DOTTED_NIC0_NET = "10.$CLUSTER.1.0";

    if ($USE_CGTP) {
      $CLUSTER_DOTTED_NIC1_NET = "10.$CLUSTER.2.0";
      $CLUSTER_DOTTED_CGTP_NET = "10.$CLUSTER.3.0";
    }
  }

  # compute the bitmask from the dotted form
  $CLUSTER_BIT_NETMASK =
    COMMON::convertDottedToBitmask($CLUSTER_DOTTED_NETMASK);
  $CLUSTER_BIT_NIC0_NET = 
    COMMON::convertDottedToBitmask($CLUSTER_DOTTED_NIC0_NET);
  if ($USE_CGTP) {
    $CLUSTER_BIT_NIC1_NET =
      COMMON::convertDottedToBitmask($CLUSTER_DOTTED_NIC1_NET);
    $CLUSTER_BIT_CGTP_NET =
      COMMON::convertDottedToBitmask($CLUSTER_DOTTED_CGTP_NET);
  }

  # reformat the dotted form from the bitmask that they had all the same
  # format
  $CLUSTER_DOTTED_NETMASK =
    COMMON::convertBitmaskToDotted($CLUSTER_BIT_NETMASK);
  $CLUSTER_DOTTED_NIC0_NET =
    COMMON::convertBitmaskToDotted($CLUSTER_BIT_NIC0_NET);
  if ($USE_CGTP) {
    $CLUSTER_DOTTED_NIC1_NET =
      COMMON::convertBitmaskToDotted($CLUSTER_BIT_NIC1_NET);
    $CLUSTER_DOTTED_CGTP_NET =
      COMMON::convertBitmaskToDotted($CLUSTER_BIT_CGTP_NET);
  }

  #information depending on network
  if ($USE_PUBLIC_NETWORK) {
    # reformat the dotted form from the bitmask that they had all the same
    # format
    $PUBLIC_BIT_NETMASK   = 
      COMMON::convertDottedToBitmask($PUBLIC_DOTTED_NETMASK);
    $PUBLIC_DOTTED_NETMASK =
      COMMON::convertBitmaskToDotted($PUBLIC_BIT_NETMASK);
    $PUBLIC_BIT_NETWORK   = 
      COMMON::convertDottedToBitmask($PUBLIC_DOTTED_NETWORK);
    $PUBLIC_DOTTED_NETWORK =
      COMMON::convertBitmaskToDotted($PUBLIC_BIT_NETWORK);

    $BIT_NETWORK   = $PUBLIC_BIT_NETWORK;
    $BIT_NETMASK   = $PUBLIC_BIT_NETMASK;
  }

  #information regarding the installation server IP address
  if ($USE_PUBLIC_NETWORK) {
    
    if (! COMMON::var_defined("SERVER_IP", "", \$SERVER_IP)) {
      print_error("SERVER_IP not defined (required when a public network is used)");
    } else {
      
      # Check the server IP address within the network
      my $bitIp = COMMON::convertDottedToBitmask($SERVER_IP);
      if (! checkIp($bitIp, $PUBLIC_BIT_NETMASK, $PUBLIC_BIT_NETWORK)) {
	print_error("server IP address $SERVER_IP: not a correct Ip address for the subnetwork $PUBLIC_DOTTED_NETWORK with netmask $PUBLIC_DOTTED_NETMASK");
      }

      # Check the default router IP address within the network
      if ($DEFAULT_ROUTER_IP ne "") {
	$bitIp = COMMON::convertDottedToBitmask($DEFAULT_ROUTER_IP);
	if (! checkIp($bitIp, $PUBLIC_BIT_NETMASK, $PUBLIC_BIT_NETWORK)) {
	  print_error("default router IP address $DEFAULT_ROUTER_IP: not a correct Ip address for the subnetwork $PUBLIC_DOTTED_NETWORK with netmask $PUBLIC_DOTTED_NETMASK");
	}
      } else {
	$DEFAULT_ROUTER_IP = $SERVER_IP;
      }
    }
    
  } else {

    # SERVER_IP ignored
    if (COMMON::var_defined("SERVER_IP", "", \$SERVER_IP)) {
      COMMON::warning("SERVER_IP ignored (only required when a public network is used)");
    }
    # DEFAULT_ROUTER_IP ignored
    if ($DEFAULT_ROUTER_IP ne "") {
      COMMON::warning("DEFAULT_ROUTER_IP ignored (only required when a public network is used)");
    }

    $BIT_NETWORK   = $CLUSTER_BIT_NIC0_NET;
    $BIT_NETMASK   = $CLUSTER_BIT_NETMASK;
    $SERVER_IP = COMMON::computeIp($SERVER_NODE, 
				   $CLUSTER_BIT_NIC0_NET,
				   $CLUSTER_BIT_NETMASK);

    # force the default router IP address with the server IP address
    $DEFAULT_ROUTER_IP = $SERVER_IP;
  }

    $DOTTED_NETMASK = COMMON::convertBitmaskToDotted($BIT_NETMASK);
  $DOTTED_NETWORK = COMMON::convertBitmaskToDotted($BIT_NETWORK);


  #check the node id  of the installation server
  if (! checkHostId($SERVER_NODE, $CLUSTER_BIT_NETMASK)) {
    print_error("Server node id $SERVER_NODE: not a correct host id for the netmask $CLUSTER_DOTTED_NETMASK");
  }

  for my $i (0..$#NODE_LIST) {

    # initialize calculated information
    if ($NODE_LIST[$i]{id} == $MASTER_ID) {
      print_error("Wrong node Id ($NODE_LIST[$i]{id}): cannot be equal to master node Id") ;
    }
    if ($NODE_LIST[$i]{id} == $SERVER_NODE) {
      print_error("Wrong node Id ($NODE_LIST[$i]{id}): already used as the installation server id")
    }
    

    # compute the IP address of each node
    if (! checkHostId($NODE_LIST[$i]{id}, $CLUSTER_BIT_NETMASK)) {
      print_error("Node $NODE_LIST[$i]{id}: not a correct host id for the netmask $CLUSTER_DOTTED_NETMASK");
    } else {

      $NODE_LIST[$i]{nic0Ip} = COMMON::computeIp($NODE_LIST[$i]{id}, 
						 $CLUSTER_BIT_NIC0_NET,
						 $CLUSTER_BIT_NETMASK);
      if ($USE_CGTP) {
	$NODE_LIST[$i]{nic1Ip} =  COMMON::computeIp($NODE_LIST[$i]{id},
						    $CLUSTER_BIT_NIC1_NET,
						    $CLUSTER_BIT_NETMASK);
	$NODE_LIST[$i]{cgtpIp} =  COMMON::computeIp($NODE_LIST[$i]{id},
						    $CLUSTER_BIT_CGTP_NET,
						    $CLUSTER_BIT_NETMASK);
      }
    }

    if ($NODE_LIST[$i]{pubConfigured}) {
      my $bitIp = COMMON::convertDottedToBitmask($NODE_LIST[$i]{pubIp});
      if (! checkIp($bitIp, $PUBLIC_BIT_NETMASK, $PUBLIC_BIT_NETWORK)) {
	print_error("Node $NODE_LIST[$i]{id}: $NODE_LIST[$i]{pubIp} not a correct Ip address for the subnetwork $PUBLIC_DOTTED_NETWORK with netmask: $PUBLIC_DOTTED_NETMASK");
	
      }
    }
  }
}

#------------------------------------------------------------------------------
#
#  check configuration
#
#------------------------------------------------------------------------------

sub check_conf
  {
    common_check ;

    if ($G_error) {
      COMMON::error("Installation aborted.");
    }

    configure_global;

    if ($G_error) {
      COMMON::error("Installation aborted.");
    }

    display_conf ;

  }

#------------------------------------------------------------------------------
#
#  load configuration
#
#  used by all sources except check_conf and diskless_found.pl
#
#------------------------------------------------------------------------------

sub load_conf
  {
    load_variables ;
    load_cluster($CLUSTER_FILE_FULLNAME) ;
    load_addon($ADDON_FILE_FULLNAME) ;

    # do not perform a global check till there is a syntax error
    if ($G_error) {
      COMMON::error("Installation aborted.");
    }
  }

{
}
