#!/usr/bin/perl
# -----------------------------------------------------------------------------
#  ident "@(#)product.pl 1.6 04/09/23 SMI"
# -----------------------------------------------------------------------------
#
#  Copyright 2001-2004 Sun Microsystems, Inc. All rights reserved.
#
# -----------------------------------------------------------------------------

package PRODUCT;

use strict;
use variables;
require "$ENV{NHINSTALL_LIB}/tools/lib/common.pl";
require "$ENV{NHINSTALL_LIB}/tools/lib/parser.pl";

#-----------------------------------------------------------------------------
#
# general variables
#
#-----------------------------------------------------------------------------

my $G_error = $FALSE;

my $COMMON_PRODUCT_FILE = "";


#-----------------------------------------------------------------------------
#
# definition of the commands
#
#-----------------------------------------------------------------------------

#
# command definitions for cluster_definition.conf
#

my $DISTRIB_TYPE = "SOLARIS,NHAS";

my @G_os_command =
(
 # COMMON_PRODUCT_FILE
 { name     => "COMMON_PRODUCT_FILE",
   category => $PARSER::CMD_SIMPLE,
   argument => [
		{
		 meaning   => "product file",
		 name      => "product",
		 type      => $PARSER::CMD_STRING,
		 variable  => \$COMMON_PRODUCT_FILE,
		 mandatory => $TRUE,
		 control   => "",
		} ,
	       ],
 } ,
 # OS
 { name     => "OS",
   category => $PARSER::CMD_HASH,
   argument => [
		{
		 meaning   => "release name",
		 name      => "name",
		 type      => $PARSER::CMD_STRING,
		 field     => "name",
		 mandatory => $TRUE,
		 control   => "",
		},
		{
		 meaning   => "OS version",
		 name      => "version",
		 type      => $PARSER::CMD_STRING,
		 field     => "version",
		 mandatory => $TRUE,
		 control   => "",
		} ,
		{
		 meaning   => "release identifier",
		 name      => "idstring",
		 type      => $PARSER::CMD_STRING,
		 field     => "idstring",
		 mandatory => $TRUE,
		 control   => "",
		} ,
		{
		 meaning   => "product file",
		 name      => "product",
		 type      => $PARSER::CMD_STRING,
		 field     => "product",
		 mandatory => $TRUE,
		 control   => "",
		} ,
		{
		 meaning   => "node profile",
		 name      => "profile",
		 type      => $PARSER::CMD_STRING,
		 field     => "profile",
		 mandatory => $TRUE,
		 control   => "",
		} ,
	       ],
   array    => \@OS_LIST,
 },
);

# requires to update array reference before parsing
my $G_location_idx  =0;
my $G_component_idx =1;

my @G_product_command =
(
 # LOCATION
 { name     => "LOCATION",
   category => $PARSER::CMD_HASH,
   argument => [
		{
		 meaning   => "name",
		 name      => "name",
		 type      => $PARSER::CMD_LIST,
		 field     => "name",
		 mandatory => $TRUE,
		 control   => $DISTRIB_TYPE,
		},
		{
		 meaning   => "package path",
		 name      => "package",
		 type      => $PARSER::CMD_STRING,
		 field     => "package",
		 mandatory => $TRUE,
		 control   => "",
		},
		{
		 meaning   => "patch path",
		 name      => "patch",
		 type      => $PARSER::CMD_STRING,
		 field     => "patch",
		 mandatory => $TRUE,
		 control   => "",
		},
		] ,
   array    => "null",
 },

 # COMPONENT
 { name     => "COMPONENT",
   category => $PARSER::CMD_HASH,
   argument => [
		{
		 meaning   => "name",
		 name      => "name",
		 type      => $PARSER::CMD_STRING,
		 field     => "name",
		 mandatory => $TRUE,
		 control   => "",
		},
		{
		 meaning   => "scope",
		 name      => "scope",
		 type      => $PARSER::CMD_LIST,
		 field     => "scope",
		 mandatory => $TRUE,
		 control   => "LOCAL,SHARED",
		},
		{
		 meaning   => "location",
		 name      => "location",
		 type      => $PARSER::CMD_LIST,
		 field     => "location",
		 mandatory => $TRUE,
		 control   => $DISTRIB_TYPE,
		},
		{
		 meaning   => "directory",
		 name      => "directory",
		 type      => $PARSER::CMD_STRING,
		 field     => "dir",
		 mandatory => $FALSE,
		 control   => "",
		},
		{
		 meaning   => "package for MEN",
		 name      => "pkg_men",
		 type      => $PARSER::CMD_STRING,
		 field     => "pkg_men",
		 mandatory => $FALSE,
		 control   => "",
		},
		{
		 meaning   => "package for dataless",
		 name      => "pkg_dataless",
		 type      => $PARSER::CMD_STRING,
		 field     => "pkg_dataless",
		 mandatory => $FALSE,
		 control   => "",
		},
		{
		 meaning   => "package for diskless root",
		 name      => "pkg_root",
		 type      => $PARSER::CMD_STRING,
		 field     => "pkg_root",
		 mandatory => $FALSE,
		 control   => "",
		},
		{
		 meaning   => "package for diskless /usr",
		 name      => "pkg_usr",
		 type      => $PARSER::CMD_STRING,
		 field     => "pkg_usr",
		 mandatory => $FALSE,
		 control   => "",
		},
		{
		 meaning   => "patch for MEN",
		 name      => "patch_men",
		 type      => $PARSER::CMD_STRING,
		 field     => "patch_men",
		 mandatory => $FALSE,
		 control   => "",
		},
		{
		 meaning   => "patch for dataless",
		 name      => "patch_dataless",
		 type      => $PARSER::CMD_STRING,
		 field     => "patch_dataless",
		 mandatory => $FALSE,
		 control   => "",
		},
		{
		 meaning   => "patch for diskless root",
		 name      => "patch_root",
		 type      => $PARSER::CMD_STRING,
		 field     => "patch_root",
		 mandatory => $FALSE,
		 control   => "",
		},
		{
		 meaning   => "patch for diskless /usr",
		 name      => "patch_usr",
		 type      => $PARSER::CMD_STRING,
		 field     => "patch_usr",
		 mandatory => $FALSE,
		 control   => "",
		},
		] ,
   array    => "null",
 },
);

#==============================================================================
#
# POST-ITEM PROCESSING
#
# USE THE PARSER ERROR FUNCTION
#
#==============================================================================


#==============================================================================
#
#  ERROR PROCESSING
#
#==============================================================================

#------------------------------------------------------------------------------
#
# print_error
#
#------------------------------------------------------------------------------

sub print_error {
  
  my ($message) = @_;
  
  $G_error = $TRUE;
  
  my ($sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst) = localtime;
  my $date = sprintf("%02.02d:%02.02d:%02.02d", $hour, $min, $sec) ;
  COMMON::print_err("$date Error: $message\n") ;
}

#==============================================================================
#
#  CONFIGURATION FILE PROCESSING
#
#==============================================================================

#------------------------------------------------------------------------------
#
#  display os description
#
#------------------------------------------------------------------------------

sub display_os {

  if (! $DEBUG_CONFIG) {
    return ;
  }
  
  COMMON::print_debug($DEBUG_CONFIG, "OS description");
  COMMON::print_debug($DEBUG_CONFIG, "--------------");

  for my $os (0..$#OS_LIST) {
    COMMON::print_debug($DEBUG_CONFIG, "");
    COMMON::print_debug($DEBUG_CONFIG, 
			"  Name:       $OS_LIST[$os]{name}");
    COMMON::print_debug($DEBUG_CONFIG, 
			"  Version:    $OS_LIST[$os]{version}");
    COMMON::print_debug($DEBUG_CONFIG, 
			"  Identifier: $OS_LIST[$os]{idstring}");
    COMMON::print_debug($DEBUG_CONFIG, 
			"  Product:    $OS_LIST[$os]{product}");
    COMMON::print_debug($DEBUG_CONFIG, 
			"  Profile:    $OS_LIST[$os]{profile}");
  }

  COMMON::print_debug($DEBUG_CONFIG, "");
}

#------------------------------------------------------------------------------
#
#  load os description
#
#  . use the user's file if existing, otherwise use the internal one
#
#------------------------------------------------------------------------------

sub load_os {

  my $filename;
  my $usr_conf_dir;

  COMMON::var_defined("NH_USR_CONF_DIR", "", \$usr_conf_dir);
  $filename = $usr_conf_dir . "/os_description.conf";
  if (! -r $filename) {
    COMMON::var_defined("NH_OS_FILE_FULLNAME", "", \$filename);
  } else {
    COMMON::warning("standard NHAS description file superseded (use of $filename).");
  }

  COMMON::print_debug($DEBUG_CONFIG, 
		"Loading OS description: $filename");
  
  if (! PARSER::parse_file(\@G_os_command, $filename)) {
    COMMON::error("Installation aborted.");
  }
  
  display_os();
}

#------------------------------------------------------------------------------
#
#  display product description
#
#------------------------------------------------------------------------------

sub display_product {


  my ($refcomp, $refloc, $node_type) = @_;

  if (! $DEBUG_CONFIG) {
    return ;
  }


  my $i;
  my $type = "";
  
  if ($node_type == $MEN_NODE) {
    $type = "MEN";
  }
  if ($node_type == $DISKLESS_NODE) {
    $type = "diskless";
  }
  if ($node_type == $DATALESS_NODE) {
    $type = "dataless";
  }

  COMMON::print_debug($DEBUG_CONFIG, "location for $type");
  COMMON::print_debug($DEBUG_CONFIG, "--------------------------------------");

  for ($i = 0; $i < @{$refloc}; $i++) {
    COMMON::print_debug($DEBUG_CONFIG, "");
    COMMON::print_debug($DEBUG_CONFIG, "    Name:    $$refloc[$i]{name}");
    COMMON::print_debug($DEBUG_CONFIG, "    Package: $$refloc[$i]{package}");
    COMMON::print_debug($DEBUG_CONFIG, "    Patch:   $$refloc[$i]{patch}");
  }

  COMMON::print_debug($DEBUG_CONFIG, "");

  COMMON::print_debug($DEBUG_CONFIG, "component for $type");
  COMMON::print_debug($DEBUG_CONFIG, "--------------------------------------");

  for ($i = 0 ; $i < @{$refcomp} ; $i++) {
    COMMON::print_debug($DEBUG_CONFIG, "");
    COMMON::print_debug($DEBUG_CONFIG, "    Name:     $$refcomp[$i]{name}");
    COMMON::print_debug($DEBUG_CONFIG, "    Scope:    $$refcomp[$i]{scope}");
    COMMON::print_debug($DEBUG_CONFIG, "    Location: $$refcomp[$i]{location}");
    COMMON::print_debug($DEBUG_CONFIG, "    Dir:      $$refcomp[$i]{dir}");

    COMMON::print_debug($DEBUG_CONFIG, "    Packages");
    if ($node_type == $MEN_NODE) {
      COMMON::print_debug($DEBUG_CONFIG, "       men:      $$refcomp[$i]{pkg_men}");
    }
    if ($node_type == $DATALESS_NODE) {
      COMMON::print_debug($DEBUG_CONFIG, "       dataless: $$refcomp[$i]{pkg_dataless}");
    }
    if ($node_type == $DISKLESS_NODE) {
      COMMON::print_debug($DEBUG_CONFIG, "       root:     $$refcomp[$i]{pkg_root}");
      COMMON::print_debug($DEBUG_CONFIG, "       usr:      $$refcomp[$i]{pkg_usr}");
    }

    COMMON::print_debug($DEBUG_CONFIG, "    Patches");

    if ($node_type == $MEN_NODE) {
      COMMON::print_debug($DEBUG_CONFIG, "       men:      $$refcomp[$i]{patch_men}");
    }
    if ($node_type == $DATALESS_NODE) {
      COMMON::print_debug($DEBUG_CONFIG, "       dataless: $$refcomp[$i]{patch_dataless}");
    }    
    if ($node_type == $DISKLESS_NODE) {
      COMMON::print_debug($DEBUG_CONFIG, "       root:     $$refcomp[$i]{patch_root}");
      COMMON::print_debug($DEBUG_CONFIG, "       usr:      $$refcomp[$i]{patch_usr}");
    }
  }

  COMMON::print_debug($DEBUG_CONFIG, "");

}

#------------------------------------------------------------------------------
#
#  look for release
#
#------------------------------------------------------------------------------

sub look_for_release {

  my ($release, $os_reference, $refidx, $ref_target_os, $refsuperseded) = @_;

  # look for the configuration file based on the Solaris release
  $$refsuperseded = $FALSE;

  for my $idx (0..$#OS_LIST) {

    if ($OS_LIST[$idx]{idstring} eq $release) {
      if ($os_reference ne "default") {
	if ($OS_LIST[$idx]{name} eq $os_reference) {

	  $$refidx = $idx;
	  $$ref_target_os = $OS_LIST[$idx]{version};
	  $$refsuperseded = $TRUE;
	  return $TRUE;

	} # else not the right key
      } else { # MEN_OS_REFERENCE not defined: used it

	$$refidx = $idx;
	$$ref_target_os = $OS_LIST[$idx]{version};
	return $TRUE;
      }
    }
  }

  return $FALSE;
}

#------------------------------------------------------------------------------
#
#  determine product file
#
#------------------------------------------------------------------------------

sub determine_product_file {

  my ($release_found, $os_idx, $user_filename_list, $ref_filename) = @_;

  my $nhinstall_conf;
  my $usr_conf_dir;

  COMMON::var_defined("NHINSTALL_CONF", "", \$nhinstall_conf);
  COMMON::var_defined("NH_USR_CONF_DIR", "", \$usr_conf_dir);

  # look for the customer-specific files: 
  foreach my $filename (split(" ",$user_filename_list)) {
    $$ref_filename = "$usr_conf_dir/${filename}";
    if (-r $$ref_filename) {
      COMMON::warning("standard product description file superseded (use of $$ref_filename).");
      return $TRUE;
    }
  }

  # look for the release based on the release string or the os reference
  if ($release_found) {
    
    # if it's a relative path, add the $NHINSTALL_CONF directory name
    if (substr($OS_LIST[$os_idx]{product},0,1) eq "/") {
      $$ref_filename = $OS_LIST[$os_idx]{product};
    } else {
      $$ref_filename = "$nhinstall_conf/" . $OS_LIST[$os_idx]{product};
    }
    return $TRUE;
  }
  
  return $FALSE;
}

#------------------------------------------------------------------------------
#
#  determine profile file
#
#------------------------------------------------------------------------------

sub determine_profile_file {

  my ($release_found, $os_idx, $user_filename_list, $ref_filename) = @_;

  my $nhinstall_conf;
  my $usr_conf_dir;

  COMMON::var_defined("NHINSTALL_CONF", "", \$nhinstall_conf);
  COMMON::var_defined("NH_USR_CONF_DIR", "", \$usr_conf_dir);

  # look for the customer-specific files: 
  foreach my $filename (split(" ",$user_filename_list)) {
    $$ref_filename = "$usr_conf_dir/${filename}";
    if (-r $$ref_filename) {
      COMMON::warning("standard Solaris profile file superseded (use of $$ref_filename).");
      return $TRUE;
    }
  }

  # look for the release based on the release string or the os reference
  if ($release_found) {
    
    # if it's a relative path, add the $NHINSTALL_CONF directory name
    if (substr($OS_LIST[$os_idx]{profile},0,1) eq "/") {
      $$ref_filename = $OS_LIST[$os_idx]{profile};
    } else {
      $$ref_filename = "$nhinstall_conf/" . $OS_LIST[$os_idx]{profile};
    }
    return $TRUE;
  }
  
  return $FALSE;
}

#------------------------------------------------------------------------------
#
#  load product description for master-eligible nodes
#
#------------------------------------------------------------------------------

sub load_men_product {

  my ($release) = @_;
  
  COMMON::print_debug($DEBUG_CONFIG, 
		      "Loading product description for master-eligible nodes");
  
  my $release_found = $FALSE;
  my $nodeprof_ok = $FALSE;
  my $product_ok = $FALSE;
  my $product_filename;
  my $superseded;

  # the node.prof.conf and product.conf based on the Solaris release
  # can be superseded by customer-specific one if there are present
  # into the configuration directory

  # moreover, the OS release detected automatically can be superseded
  # by the customer-specific OS used as key

  # look for the release based on the release string or the os reference
  $release_found = look_for_release($release, $MEN_OS_REFERENCE, 
				    \$MEN_OS_IDX, \$MEN_TARGET_OS,
				    \$superseded) ;

  # look for the configuration file based on the Solaris release

  $product_ok = determine_product_file($release_found,
				       $MEN_OS_IDX,
				       "product.conf",
				       \$product_filename
				      );
  $nodeprof_ok = determine_profile_file($release_found,
					$MEN_OS_IDX,
				       "nodeprof.conf",
				       \$MEN_PROFILE
				      );

  # are all files defined ?
  # maybe not for an unknown release and if the customer does not
  # provided all configuration files.
  if (((! $product_ok) || (! $nodeprof_ok)) && (! $release_found)) {
    if ($MEN_OS_REFERENCE eq "default") {
      COMMON::error("Unsupported Solaris release ($release): cannot load product information");
    } else {
      COMMON::error("Unsupported OS reference ($MEN_OS_REFERENCE): cannot load product information");
    }
  }

  if ($superseded) {
    COMMON::warning("Solaris detection for master-eligible nodes superseded: using $MEN_OS_REFERENCE");
  }

  COMMON::print_debug($DEBUG_CONFIG, "Loading $product_filename ...");

  $G_product_command[$G_location_idx]{array} = \@MEN_LOCATION;
  $G_product_command[$G_component_idx]{array} = \@MEN_COMPONENT;
  if ($COMMON_PRODUCT_FILE ne "") {
    my $nhinstall_conf;
    COMMON::var_defined("NHINSTALL_CONF", "", \$nhinstall_conf);
    if (! PARSER::parse_file(\@G_product_command, 
			     "$nhinstall_conf/" . $COMMON_PRODUCT_FILE)) {
      COMMON::error("Installation aborted.");
    }
  }
  if (! PARSER::parse_file(\@G_product_command, $product_filename)) {
    COMMON::error("Installation aborted.");
  }

  display_product(\@MEN_COMPONENT, \@MEN_LOCATION, $MEN_NODE);

  COMPONENT::init_men_distrib();
}

#------------------------------------------------------------------------------
#
#  load product description for diskless nodes
#
#------------------------------------------------------------------------------

sub load_diskless_product {

  my ($release) = @_;
  
  COMMON::print_debug($DEBUG_CONFIG,
		      "Loading product description for diskless") ;

  my $release_found = $FALSE;
  my $product_ok = $FALSE;
  my $nodeprof_ok = $FALSE;
  my $product_filename;
  my $superseded;

  # the product.conf based on the Solaris release
  # can be superseded by customer-specific one if there are present
  # into the configuration directory
  # moreover, the OS release detected automatically can be superseded
  # by the customer-specific OS used as key

  # look for the release based on the release string or the os reference
  $release_found = look_for_release($release, $DISKLESS_OS_REFERENCE, 
				    \$DISKLESS_OS_IDX, \$DISKLESS_TARGET_OS,
				    \$superseded) ;

  # look for the configuration file based on the Solaris release

  $product_ok = determine_product_file($release_found,
					$DISKLESS_OS_IDX,
				       "diskless_product.conf product.conf",
				       \$product_filename
				      );
  $nodeprof_ok = determine_profile_file($release_found,
					$DISKLESS_OS_IDX,
				       "diskless_nodeprof.conf nodeprof.conf",
				       \$DISKLESS_PROFILE
				      );

  # are all files defined ?
  # maybe not for an unknown release and if the customer does not
  # provided all configuration files.
  if ((! $product_ok) && (! $release_found)) {
    if ($DISKLESS_OS_REFERENCE eq "default") {
      COMMON::error("Unsupported Solaris release ($release): cannot load product information");
    } else {
      COMMON::error("Unsupported OS reference ($DISKLESS_OS_REFERENCE): cannot load product information");
    }
  }

  if ($superseded) {
    COMMON::warning("Solaris detection for diskless superseded: using $DISKLESS_OS_REFERENCE");
  }
  COMMON::print_debug($DEBUG_CONFIG, "Loading $product_filename ...");

  $G_product_command[$G_location_idx]{array} = \@DISKLESS_LOCATION;
  $G_product_command[$G_component_idx]{array} = \@DISKLESS_COMPONENT;
  if ($COMMON_PRODUCT_FILE ne "") {
    my $nhinstall_conf;
    COMMON::var_defined("NHINSTALL_CONF", "", \$nhinstall_conf);
    if (! PARSER::parse_file(\@G_product_command,
			     "$nhinstall_conf/" . $COMMON_PRODUCT_FILE)) {
      COMMON::error("Installation aborted.");
    }
  }
  if (! PARSER::parse_file(\@G_product_command, $product_filename)) {
    COMMON::error("Installation aborted.");
  }

  display_product(\@DISKLESS_COMPONENT, \@DISKLESS_LOCATION, $DISKLESS_NODE);

  COMPONENT::init_diskless_distrib();
}

#------------------------------------------------------------------------------
#
#  load product description for dataless nodes
#
#------------------------------------------------------------------------------

sub load_dataless_product {

  my ($release) = @_;
  
  COMMON::print_debug($DEBUG_CONFIG,
		      "Loading product description for dataless") ;

  my $release_found = $FALSE;
  my $product_ok = $FALSE;
  my $nodeprof_ok = $FALSE;
  my $product_filename;
  my $superseded;

  # the product.conf based on the Solaris release
  # can be superseded by customer-specific one if there are present
  # into the configuration directory
  # moreover, the OS release detected automatically can be superseded
  # by the customer-specific OS used as key

  # look for the release based on the release string or the os reference
  $release_found = look_for_release($release, $DATALESS_OS_REFERENCE, 
				    \$DATALESS_OS_IDX, \$DATALESS_TARGET_OS,
				    \$superseded) ;

  # look for the configuration file based on the Solaris release

  $product_ok = determine_product_file($release_found,
					$DATALESS_OS_IDX,
				       "dataless_product.conf product.conf",
				       \$product_filename
				      );
  $nodeprof_ok = determine_profile_file($release_found,
					$DATALESS_OS_IDX,
				       "dataless_nodeprof.conf nodeprof.conf",
				       \$DATALESS_PROFILE
				      );

  # are all files defined ?
  # maybe not for an unknown release and if the customer does not
  # provided all configuration files.
  if ((! $product_ok) && (! $release_found)) {
    if ($DATALESS_OS_REFERENCE eq "default") {
      COMMON::error("Unsupported Solaris release ($release): cannot load product information");
    } else {
      COMMON::error("Unsupported OS reference ($DATALESS_OS_REFERENCE): cannot load product information");
    }
  }

  if ($superseded) {
    COMMON::warning("Solaris detection for dataless superseded: using $DATALESS_OS_REFERENCE");
  }
  COMMON::print_debug($DEBUG_CONFIG, "Loading $product_filename ...");

  $G_product_command[$G_location_idx]{array} = \@DATALESS_LOCATION;
  $G_product_command[$G_component_idx]{array} = \@DATALESS_COMPONENT;
  if ($COMMON_PRODUCT_FILE ne "") {
    my $nhinstall_conf;
    COMMON::var_defined("NHINSTALL_CONF", "", \$nhinstall_conf);
    if (! PARSER::parse_file(\@G_product_command,
			     "$nhinstall_conf/" . $COMMON_PRODUCT_FILE)) {
      COMMON::error("Installation aborted.");
    }
  }
  if (! PARSER::parse_file(\@G_product_command, $product_filename)) {
    COMMON::error("Installation aborted.");
  }

  display_product(\@DATALESS_COMPONENT, \@DATALESS_LOCATION, $DATALESS_NODE);

  COMPONENT::init_dataless_distrib();
}

{
}
