#!/usr/bin/perl -w
# ------------------------------------------------------------------------------
#   ident "@(#)sl_jumpstart.pl 1.15     03/09/09 SMI"
# ------------------------------------------------------------------------------

BEGIN { 
   require "$ENV{'NHAS_PROD_DIR'}/nhsmct/lib/sl_utils.pl"; 
   require "$ENV{'NHAS_PROD_DIR'}/nhsmct/lib/sl_registry.pl"; 
}

# ------------------------------------------------------------------------------

package sl_jumpstart;

# Target machine to install
%JUMPSTART_TARGET = ( 'PROTOTYPE' => "proto",
                      'CLUSTER' => "cluster" );

# Make the saaociation between the Solaris release and the directory where
# release dependent files are stored
%JUMPSTART_RELEASE_TO_DIR = 
   ( 'Solaris 9 s9_58shwpl3 SPARC' => "Solaris_9",
     'Solaris 8 2/02 s28s_u7wos_08a SPARC' => "Solaris_8",
     'Solaris 9 9/02 s9s_u1wos_08b SPARC' => "Solaris_9u1",
     'Solaris 8 HW 12/02 s28s_hw1wos_06a SPARC' => "Solaris_8psr1",
     'Solaris 8 HW 7/03 s28s_hw3wos_05a SPARC' => "Solaris_8psr3" );

# Jumpstart Destination directory
$JUMPSTART_DIR = "jumpstart";
$JUMPSTART_INSTALL_DIR = "${JUMPSTART_DIR}/install";
$JUMPSTART_INSTALL_PKG_DIR = "${JUMPSTART_INSTALL_DIR}/pkg";
$JUMPSTART_INSTALL_PAT_DIR = "${JUMPSTART_INSTALL_DIR}/pat";
$JUMPSTART_INSTALL_SW_DIR = "${JUMPSTART_INSTALL_DIR}/sw";
$JUMPSTART_DEPLOY_DIR = "${JUMPSTART_DIR}/deploy";
$JUMPSTART_DEPLOY_SCRIPT_DIR = "${JUMPSTART_DEPLOY_DIR}/scripts";

# Jumpstart configuration files names
my $JUMPSTART_PROFILE = "profile";
my $JUMPSTART_RULES = "rules";
my $JUMPSTART_BEGIN = "begin";
my $JUMPSTART_FINISH = "finish";
my $JUMPSTART_SYSIDCFG = "sysidcfg";
my $JUMPSTART_DISK_LAYOUT = "disk_layout";

# Solaris cluster toc file
my $CLUSTER_TOC = ".clustertoc";

# Solaris release data
my $RELEASE_FILE = "release";
my $RELEASE_DIR = "SUNWsolnm/reloc/etc";
my $COMMITED_RELEASES = "release";

# ------------------------------------------------------------------------------
#
sub getSolarisDir {
   &sl_traces::trace( 2, "- sl_jumpstart::getSolarisDir - Getting ".
                      "Solaris product directory" );
   my $productName = &getProductName;
   return "$ENV{'SMCT_SOL_DIR'}/${productName}/Product";
}

# ------------------------------------------------------------------------------
#
sub getSolarisEA {
   &sl_traces::trace( 2, "- sl_jumpstart::getSolarisEA - Getting ".
                      "Solaris Early Access directory" );
   my $productName = &getProductName;
   return "$ENV{'SMCT_SOL_DIR'}/${productName}/EA";
}

# ------------------------------------------------------------------------------
#
sub getProductName {
   &sl_traces::trace( 2, "- sl_jumpstart::getProductName - Getting ".
                      "Solaris product name" );
   my $productName = 
      `$ENV{'SMCT_LS'} $ENV{'SMCT_SOL_DIR'} | $ENV{'SMCT_GREP'} Solaris`;
   chop( $productName );
   return $productName;
}

# ------------------------------------------------------------------------------
#
sub getSolarisName {
   &sl_traces::trace( 2, "- sl_jumpstart::getSolarisName - Getting ".
                      "Solaris product name" );
   my $release = &checkSolarisRelease;
   $release =~ s/^(\s)+//;
   my $productName = $JUMPSTART_RELEASE_TO_DIR{$release};
   return $productName;
}

# ------------------------------------------------------------------------------
#
sub getSolarisRelease {
   &sl_traces::trace( 2, "- sl_jumpstart::getSolarisRelease - Getting ".
                      "Solaris release" );
   my $solarisDir = &getSolarisDir;
   my $releaseDir = "${solarisDir}/${RELEASE_DIR}";
   my $releaseFile = "${releaseDir}/${RELEASE_FILE}";
   # Retrieve release from the /etc/release file
   open( RELEASE , $releaseFile ) ||
      &sl_traces::error( "Cannot open < %s >", 
			 "- sl_jumpstart::getSolarisRelease - ".__LINE__,
			 ( $releaseFile ) );
   my @releaseData = <RELEASE>;
   close( RELEASE );
   my $release = $releaseData[0];
   chop( $release );
   return $release;
}

# ------------------------------------------------------------------------------
#
sub checkSolarisRelease {
   &sl_traces::trace( 2, "- sl_jumpstart::checkSolarisRelease - Checking ".
                      "Solaris release" );
   my $curRelease = &getSolarisRelease;
   my $comReleaseFile = "jumpstart/${COMMITED_RELEASES}";
   $comReleaseFile = &sl_utils::selectFile( $comReleaseFile, "", "mandatory" );
   open( COM_RELEASE , $comReleaseFile ) ||
      &sl_traces::error( "Cannot open < %s >", 
			 "- sl_jumpstart::checkSolarisRelease - ".__LINE__,
			 ( $comReleaseFile ) );
   my @comReleaseData = <COM_RELEASE>;
   close( COM_RELEASE );
   my $supRelease = "\n";
   foreach $release ( @comReleaseData ) {
      chop( $release );
      next if ( $release =~ /^#/ );
      return $curRelease if ( $curRelease =~ /${release}/ );
      $supRelease .= $release."\n"; 
   }
   &sl_traces::error( 
      "The Solaris release\n\n\"%s\"\n\nis not supported. ".
      "Supported releases are:\n%s", 
      "- sl_jumpstart::checkSolarisRelease - ".__LINE__,
      ( $curRelease, $supRelease ) );
}

# ------------------------------------------------------------------------------
#
sub _checkProfile {
   my ( $target, $profile ) = @_;
   &sl_traces::trace( 3, "- sl_jumpstart::_checkProfile - Checking profile ".
                      "< $profile >" );
   open ( PROF, "<".$profile ) ||
      &sl_traces::error( "Cannot open < %s >", 
			 "- sl_jumpstart::_checkProfile - ".__LINE__,
			 ( $profile ) );
   @profileData = <PROF>;
   close PROF;
   my $productDir = &getSolarisDir;
   foreach ( @profileData ) {
      chomp;
      if ( /^cluster/ ) {
         # Check cluster existence
         /^cluster\s+(\S+)/;
	 if ( defined $1 ) {
	    $pattern = "CLUSTER=${1}";
	    $command = "$ENV{'SMCT_CAT'} ${productDir}/${CLUSTER_TOC} |".
	               "$ENV{'SMCT_GREP'} ${pattern}";
            $cluster = `$command`;
	    chomp( $cluster );
	    unless ( $cluster =~ /${pattern}/ ) {
	       &sl_traces::error( "Cluster ".
	                          "< %s > has not been found\nin the ".
			          "cluster toc < %s >".
			          "\nCurrent cluster definition is < %s >",
				  "- sl_jumpstart::_checkProfile - ".__LINE__,
				  ( $1, "${productDir}/${CLUSTER_TOC}", $_ ) );
	    }
	 }
	 else {
	    &sl_traces::error( "Cluster is ".
	                       "not defined in the profile < %s >\n".
			       "Current cluster definition is < %s >", 
			       "- sl_jumpstart::_checkProfile - ".__LINE__,
			       ( $profile, $_ ) );
	 }
      }
      elsif ( /^package/ ) {
         # Check package existence
         /^package\s+(\S+)/;
	 if ( defined $1 ) {
	    unless ( ( -f "${productDir}/${1}/pkgmap" ) or
	             ( -f "${productDir}/${1}\.u/pkgmap" ) ) {
	       &sl_traces::error( "Package ".
	                          "< %s > has not been found\nin the ".
			          "< %s >\nCurrent package ".
			          "product definition is < %s >", 
				  "- sl_jumpstart::_checkProfile - ".__LINE__,
				  ( $1, $productDir, $_ ) );
	    }
	 }
	 else {
	    &sl_traces::error( "Package is ".
	                       "not defined in the profile < %s >\n".
			       "Current package definition is < %s >", 
			       "- sl_jumpstart::_checkProfile - ".__LINE__,
			       ( $profile, $_ ) );
	 }
      }
   }
}

# ------------------------------------------------------------------------------
#
sub _getDiskLayout {
   my ( $groupName, $node, $clusterName, $swlVersion, @diskLst ) = @_;
   &sl_traces::trace( 3, "- sl_jumpstart::_getDiskLayout - Getting node ".
                      "disk(s) layout" );
   my $layout = "";
   foreach $disk ( @diskLst ) {
      $diskName = $disk->getDevice;
      @sliceLst = $disk->getSliceLst;
      $layout .= "usedisk ${diskName}\n";
      my %sliceNumberLst = ();
      my %sliceSizeLst = ();
      foreach $slice ( @sliceLst ) {
	 my $sliceNumber = $slice->getNumber;
	 my $sliceSize = $slice->getSize;
	 my $sliceName = $slice->getName;
	 if ( $slice->isPhys ) {
	    $sliceNumberLst{$sliceName} = $sliceNumber;
	    $sliceSizeLst{$sliceName} = $sliceSize;
	 }
      }
      foreach $slice ( @sliceLst ) {
         my $fileSys = $slice->getFileSys;
	 my $sliceName = $slice->getName;
	 my $physSliceName = $slice->getPhysSlice;
	 my $sliceKey = ( $slice->isPhys ) ? $sliceName:$physSliceName;
	 if ( defined $fileSys ) {
	    my $partition = "${diskName}s$sliceNumberLst{$sliceKey}";
	    # Remove phys-slice from phys slice list
	    $sliceNumberLst{$sliceKey} = undef;
  	    my @exportedFileSysLst = $fileSys->getExportedFileSysLst;
	    my $role = $fileSys->getRole;
            my $type = $fileSys->getType;
	    my $size = $slice->getSize;
	    my $mntOpt = $fileSys->getMntOpt;
	    my $mntPoint = $fileSys->getMntPoint;
	    if ( $type eq ${sl_filesys::FILESYS_TYPE{'SWAP'}} ) {
	       $layout .=
		  "filesys $partition ${size} ${type}\n";
	    }
	    else {
	       if ( $role eq ${sl_filesys::FILESYS_ROLE{'ROOT'}} ) {
		  $layout .= 
	             "boot_device $partition update\n".
		     "root_device $partition\n";
	       }
	       $layout = $layout.
	          "filesys $partition ".
		  "${size} ${mntPoint} ${mntOpt}\n";
	    }
	    # Catch mount point of the UFS mandatory filesys
	    if ( $role eq ${sl_filesys::FILESYS_ROLE{'EXPORT'}} ) {
               &sl_registry::setValue( 
	          $groupName, "EXPORT_ROOT", $mntPoint );
               &sl_registry::setValue( 
	          $clusterName, "EXPORT_ROOT", $mntPoint );
	    }
	    elsif ( $role eq ${sl_filesys::FILESYS_ROLE{'SHARED'}} ) {
               &sl_registry::setValue( 
	          $groupName, "SHARED_ROOT", $mntPoint );
               &sl_registry::setValue( 
	          $clusterName, "SHARED_ROOT", $mntPoint );
	    }
	    # Catch mount point of the NFS filesys
	    foreach $exportedFileSys ( @exportedFileSysLst ) {
	       $role = $exportedFileSys->getRole;
	       $mntPoint = $exportedFileSys->getMntPoint;
	       $remMntPoint = $exportedFileSys->getRemMntPoint;
	       $mntOpt = $exportedFileSys->getMntOpt;
	       if ( $role eq ${sl_filesys::FILESYS_ROLE{'SERVICES'}} ) {
	          $mntPoint =~ s/${sl_filesys::SWL_TAG}/${swlVersion}/g;
        	  &sl_registry::setValue( 
		     $groupName, "SERVICES_ROOT", $mntPoint );
        	  &sl_registry::setValue( 
		     $clusterName, "SERVICES_ROOT", $mntPoint );
		  &sl_registry::setValue(
		     $clusterName, "SERVICES_REMOTE", $remMntPoint );
		  &sl_registry::setValue(
		     $clusterName, "SERVICES_MNT_OPT", $mntOpt );
	       }
	       elsif ( $role eq ${sl_filesys::FILESYS_ROLE{'DATABASE'}} ) {
        	  &sl_registry::setValue( 
		     $groupName, "DATABASE_ROOT", $mntPoint );
        	  &sl_registry::setValue( 
		     $clusterName, "DATABASE_ROOT", $mntPoint );
		  &sl_registry::setValue(
		     $clusterName, "DATABASE_REMOTE", $remMntPoint );
		  &sl_registry::setValue(
		     $clusterName, "DATABASE_MNT_OPT", $mntOpt );
	       }
	       elsif ( $role eq ${sl_filesys::FILESYS_ROLE{'DATA'}} ) {
        	  &sl_registry::setValue( 
		     $groupName, "DATA_ROOT", $mntPoint );
        	  &sl_registry::setValue( 
		     $clusterName, "DATA_ROOT", $mntPoint );
		  &sl_registry::setValue(
		     $clusterName, "DATA_REMOTE", $remMntPoint );
		  &sl_registry::setValue(
		     $clusterName, "DATA_MNT_OPT", $mntOpt );
	       }    
	    }
         }
      }
      # Add remaining physical slice not configured for file systems
      # (metaDB slice, bitmap slice, underlying soft partition slice ...)
      foreach $sliceKey (  keys %sliceNumberLst ) {
        if ( defined $sliceNumberLst{$sliceKey} ) {
	    my $partition = "${diskName}s$sliceNumberLst{$sliceKey}";
	    $layout .=
	       "filesys ${partition} $sliceSizeLst{$sliceKey}\n";
	}
      }
   }
   return $layout;
}

# ------------------------------------------------------------------------------
#
sub _makeProfile {
   my ( $target, $profile, $destDir, $group, 
        $swlVersion, $clusterName, @shelfLst ) = @_;
   &sl_traces::trace( 3, "- sl_jumpstart::_makeProfile - Making profile ".
                     "< ${profile} > " );
   &_checkProfile( $target, $profile );
   ( -d $destDir ) || &sl_utils::createDir( $destDir );
#   if ( $target eq $JUMPSTART_TARGET{'PROTOTYPE'} ) {
   # Retrieve filesys information from node disk definition
   my @nodeLst = $group->getNodeLst;
   my $groupName = $group->getName;
   my $groupType = $group->getType;
   # Take first node of the group as reference node
   my $nodeRef = $nodeLst[0];
   my $nodeRefOid = $nodeRef->iD;
   my @nodeRefDiskOidLst = $nodeRef->getDiskOidLst;
   my @nodeRefDiskLst = ();
   # Disk(s) look-up: the targeted drawer is the one holding the MEN disk 
   # (this is not necessarily the same as the one holding the MEN board) 
   my @diskLst = ();
   foreach $shelf ( @shelfLst ) {
      push ( @diskLst, $shelf->getDiskLst );
   }
   # Build the disk list for the reference node
   foreach $disk ( @diskLst ) {
      foreach $diskOid ( @nodeRefDiskOidLst ) {    
         if ( $diskOid eq $disk->getName ) {
	    push( @nodeRefDiskLst, $disk );
         }
      } 
   }
   &sl_traces::error( "No disk are defined ".
                      "for the node < %s >".
		      "\nNode group < %s >".
		      "\nNode role < %s >", 
		      "- sl_jumpstart::_makeProfile - ".__LINE__,
		      ( $nodeRefOid, $groupName, $groupType ) )
      unless ( scalar @nodeRefDiskLst != 0 );
   # Build the disk layout of the reference node
   my $layoutRef = &_getDiskLayout( $groupName, $nodeRef, $clusterName,
                                    $swlVersion, @nodeRefDiskLst );
   # Build the disk list for the other nodes of the group and compare to 
   # the layout of the reference node  
   for ( $i=1; $i < scalar @nodeLst; $i++ ) {
      $node = $nodeLst[$i];
      $nodeOid = $node->iD;
      @nodeDiskOidLst = $nodeRef->getDiskOidLst;
      @nodeDiskLst = ();
      # Build the disk list for the current node
      foreach $disk ( @diskLst ) {
	 foreach $diskOid ( @nodeDiskOidLst ) {    
            if ( $diskOid eq $disk->getName ) {
	       push( @nodeDiskLst, $disk );
            }
	 } 
      }
      &sl_traces::error( "No disk are defined ".
                	 "for the node < %s >".
			 "\nNode group < %s >".
			 "\nNode role < %s >", 
			 "- sl_jumpstart::_makeProfile - ".__LINE__,
			 ( $nodeOid, $groupName, $groupType ) )
	 unless ( scalar @nodeDiskLst != 0 );
      # Build the disk layout of the current node
      $layout = &_getDiskLayout( $groupName, $node, $clusterName, 
                                 $swlVersion, @nodeDiskLst );
      if ( $layout ne $layoutRef ) {
         &sl_traces::error( "Disks of group ".
	                    "< %s > have different disk(s) ".
		            "layouts\n\nnode < %s > disk(s) ".
		            "layout(s):\n%s\n".
		            "node < %s > disk(s) layout(s):\n".
			    "%s",
			    "- sl_jumpstart::_makeProfile - ".__LINE__,
			    ( $groupName, $nodeRefOid, $layoutRef, 
			      $nodeOid, $layout ) );
      }   
   }
   # Append disk layout to Jumpstart profile template
   my $command = "$ENV{'SMCT_CAT'} ${profile} ".
                 " > ${destDir}/${JUMPSTART_PROFILE};".
        	 "echo \"${layoutRef}\" ".
                 " >> ${destDir}/${JUMPSTART_PROFILE};";
   &sl_utils::execCmd( $command );
   # Store disk layout for deployment stage
   $command = "echo \"${layoutRef}\" > ${destDir}/${JUMPSTART_DISK_LAYOUT}";
   &sl_utils::execCmd( $command );
   # Remove boot_device root_device and usedisk from the deployment profile
   # (not compatible with install_type=flash_install)
   if ( $target eq $JUMPSTART_TARGET{'CLUSTER'} ) {
      # Disk layout has been calculated by slcreate operation
      my $diskLayout = "${destDir}/${JUMPSTART_DISK_LAYOUT}";
      my $command = "$ENV{'SMCT_CAT'} ${profile} ".
                    " > ${destDir}/${JUMPSTART_PROFILE};".
                    "$ENV{'SMCT_CAT'} ${diskLayout} ".
		    "| $ENV{'SMCT_GREP'} -v boot_device ".
		    "| $ENV{'SMCT_GREP'} -v root_device ".
		    "| $ENV{'SMCT_GREP'} -v usedisk ".
                    " >> ${destDir}/${JUMPSTART_PROFILE};";
      &sl_utils::execCmd( $command );
   }
}

# ------------------------------------------------------------------------------
#
sub _makeNodeConf {
   my ( $group, $destDir, @shelfLst ) = @_;
   &sl_traces::trace( 3, "- sl_jumpstart::_makeNodeConf - Making nodes ".
                      "configuration file" );
   # Create Jumpstart node config files
   my @nodeLst = $group->getNodeLst;
   foreach $node ( @nodeLst ) {
      my $nodeId = $node->getId;
      my $nodeName = $node->getName;
      my $drawerId = $node->getDrawer;
      my $drawer = undef;
      foreach $shelf ( @shelfLst ) {
	 $drawer = $shelf->getDrawer( $drawerId );
	 last if (defined $drawer);
      }
      my $board = $drawer->getBoard( $node->getBoard );
      my @nicLst = $board->getNicLst;
      my $ipAddress = undef;
      my $mac = undef;
      my $nicName = undef;
      # Look for primary network IF
      foreach $nic ( @nicLst ) {
         my $class = $nic->getClass;
	 if ( $class eq ${sl_nic::NIC_CLASS{'PRIMARY'}} ) { 
	    $ipAddress = $nic->getIpAddress;
	    $mac = $nic->getAddress;
	    $nicName = $nic->getName;
	    last;
	 }
      }
      my $ip = $ipAddress->getAddress;
      my $host = $ipAddress->getName;
      &sl_traces::error( "The following parameter of the node ".
                         "< %s > is undefined:\n%s",
			 "- sl_jumpstart::makeNodeConf -".__LINE__,
			 ( $nodeName, "ip address" ) )
         unless ( defined $ip );
      &sl_traces::error( "The following parameter of the node ".
                         "< %s > is undefined:\n%s",
			 "- sl_jumpstart::makeNodeConf -".__LINE__,
			 ( $nodeName ,"${nicName} ethernet mac address" ) )
         unless ( defined $mac );
      my $deployConf = "${destDir}/clone-system.${nodeId}.${nodeName}.conf";
      my $command = "echo \"name ${host}\" > ${deployConf};".
	            "echo \"ip ${ip}\" >> ${deployConf};".
	            "echo \"mac ${mac}\" >> ${deployConf};";
      &sl_utils::execCmd( $command );
   }
}

# ------------------------------------------------------------------------------
#
sub configure {
   my ( $target, $destDir, $group, $swlVersion, $clusterName, @shelfLst ) = @_;
   my $groupName = $group->getName;
   &sl_traces::trace( 2, "- sl_jumpstart::configure - Configuring Jumpstart ".
                      "for group < ${groupName} >" );
   # Determine os targetted
   my $groupOs = $group->getOs;
   &sl_traces::error( "Cannot configure Jumpstart for the nodes group ".
                      "< %s >\nNodes group OS must be < %s >".
		      "\nCurrent OS value is: %s",
		      "- sl_jumpstart::configure - ".__LINE__,
		      ( $groupName, $sl_node::NODE_OS{'SOLARIS'}, $groupOS ) )
      unless ( $groupOs eq $sl_node::NODE_OS{'SOLARIS'} );
   &checkSolarisRelease;
   my $solarisDir = &getSolarisName;
   # Look for Jumpstart configuration files
   my $profile = ( $target eq $JUMPSTART_TARGET{'PROTOTYPE'} ) ?
       "${solarisDir}/${JUMPSTART_PROFILE}.${target}.${groupName}":
       "${JUMPSTART_PROFILE}.${target}.${groupName}";
   $profile =~ m,(.+).${target}\.([^\.].+),;
   my $profileDef = "${JUMPSTART_DIR}/${1}.${target}";
   $profile = &sl_utils::selectFile( $profile, $profileDef, "mandatory" );  
   my $begin = "${JUMPSTART_BEGIN}.${target}.${groupName}";
   $begin =~ m,(.+).${target}\.([^\.].+),;
   my $beginDef = "${JUMPSTART_DIR}/${1}.${target}";
   $begin = &sl_utils::selectFile( $begin, $beginDef );  
   my $finish = "${JUMPSTART_FINISH}.${target}.${groupName}";
   $finish =~ m,(.+).${target}\.([^\.].+),;
   my $finishDef = "${JUMPSTART_DIR}/${1}.${target}";
   $finish = &sl_utils::selectFile( $finish, $finishDef, "mandatory" );  
   my $sysidcfg = "${JUMPSTART_SYSIDCFG}.${target}.${groupName}";
   $sysidcfg =~ m,(.+).${target}\.([^\.].+),;
   my $sysidcfgDef = "${JUMPSTART_DIR}/${1}.${target}";
   $sysidcfg = &sl_utils::selectFile( $sysidcfg, $sysidcfgDef, "mandatory" );  
   my $rules = "${JUMPSTART_RULES}.${target}.${groupName}";
   $rules =~ m,(.+).${target}\.([^\.].+),;
   my $rulesDef = "${JUMPSTART_DIR}/${1}.${target}";
   $rules = &sl_utils::selectFile( $rules, $rulesDef, "mandatory" );  
   # Generate the Jumpstart profile file
   &_makeProfile(
      $target, $profile, $destDir, $group, 
      $swlVersion, $clusterName, @shelfLst );
   # Copy predefined Jumpstart config. files into dest. dir 
   my $command = "";
   foreach $file ( $begin, $finish, $sysidcfg, $rules ) {
      $file =~ m,/(.+)/([^/].+)\.${target},;
      $command .= "$ENV{'SMCT_CP'} ${file} ${destDir}/${2};"
         if ( -f $file );
   }
   &sl_utils::execCmd( $command );
   if ( $target eq $JUMPSTART_TARGET{'CLUSTER'} ) {
      &_makeNodeConf( $group, $destDir, @shelfLst );
   }
}

# ------------------------------------------------------------------------------

return 1;
